@@ -1,4 +1,5 @@
 #!/bin/sh
+# A Kino export script that outputs to H.264 MP4 using ffmpeg with x264
 
 usage()
 {
@@ -6,17 +7,19 @@ usage()
 	echo "Title: H.264 MP4 Single Pass (FFMPEG)"
 
 	# Usable?
-	aac=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*E.*h264)" | grep h264 | wc -l`
-	[ "$aac" -gt 0 ] && echo Status: Active || echo Status: Inactive
+	aac=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EA.*aac)" | grep aac | wc -l`
+	h264=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EV.*264)" | grep 264 | wc -l`
+	[ "$aac" -gt 0 ] && [ "$h264" -gt 0 ] && echo Status: Active || echo Status: Inactive
 
 	# Type
 	echo Flags: single-pass file-producer
 
 	# Profiles
-	echo "Profile: High Quality (640x480, VBR)"
-	echo "Profile: Medium Quality (320x240, VBR)"
-	echo "Profile: Broadband Quality (320x240, 364 kb/s)"
-	echo "Profile: 3G Mobile Quality (176x144, 12fps, 128 kb/s)"
+	echo "Profile: Best Quality (native size, interlace, VBR)"
+	echo "Profile: High Quality (full size, progressive, VBR)"
+	echo "Profile: Medium Quality (medium size, progressive, VBR)"
+	echo "Profile: Broadband Quality (medium size, progressive, 564 kb/s)"
+	echo "Profile: 3G Mobile Quality (176x144, progressive, 12fps, 128 kb/s)"
 }
 
 execute()
@@ -28,9 +31,14 @@ execute()
 	file="$4"
 	project="$5"
 	pass="$6"
+	aspect="$7"
 
-	source "`dirname $0`/ffmpeg_utils.sh"
+	. "`dirname $0`/ffmpeg_utils.sh"
 	ffmpeg_generate_hq
+	test_lf=`echo $ffmpeg_help | grep '\-lf' | wc -l`
+	[ "$test_lf" -gt 0 ] && hq="$hq -lf"
+	test_flags=`echo $ffmpeg_help | grep '\-flags' | wc -l`
+	[ "$test_flags" -gt 0 ] && hq="$hq -flags +loop"
 
 	# generate filename if missing
 	[ "x$file" = "x" ] && file="kino_export_"`date +%Y-%m-%d_%H.%M.%S`
@@ -44,19 +52,24 @@ execute()
 	# Run the command
 	case "$profile" in 
 		"0" ) 	ffmpeg -f dv -i pipe: \
-			-vcodec h264 $hq -s 640x480 -aspect 4:3 -qscale 4 -ab 120 \
+			-vcodec h264 -g 300 $hq $interlace -aspect $aspect -qscale 2 -ab 192$audio_kilo \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"1" ) 	ffmpeg -f dv -i pipe: \
-			-vcodec h264 $hq -s 320x240 -aspect 4:3 -deinterlace -qscale 4 -ab 64 -ar 44100 \
+			-vcodec h264 -g 300 $hq $progressive -s $full_res -aspect $aspect -qscale 2 -ab 192$audio_kilo \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"2" ) 	ffmpeg -f dv -i pipe: \
-			-vcodec h264 $hq -s 320x240 -aspect 4:3 -deinterlace -b 300 -g 60 -ab 32 -ar 22050 \
+			-vcodec h264 -g 300 $hq $progressive -s $med_res -aspect $aspect -qscale 4 -ab 128$audio_kilo -ar 44100 \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"3" ) 	ffmpeg -f dv -i pipe: \
-			-vcodec h264 $hq -r 12 -g 36 -s qcif -aspect 4:3 -deinterlace -b 100 -ac 1 -ab 28 -ar 16000 \
+			-vcodec h264 -g 300 $hq $progressive -s $med_res -aspect $aspect -b 500$kilo -ab 64$audio_kilo -ar 32000 \
+			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
+			-y "$file".mp4 ;;
+		"4" ) 	ffmpeg -f dv -i pipe: \
+			-vcodec h264 $hq -r 12 -g 120 $progressive -s qcif -aspect $aspect -b 90$kilo -minrate 0 -maxrate 100$kilo -bufsize 40$bufsize \
+			-ac 1 -ab 28$audio_kilo -ar 16000 \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 	esac
