@@ -1,4 +1,5 @@
 #!/bin/sh
+# A Kino export script that outputs to MP3 using ffmpeg with lame
 
 usage()
 {
@@ -8,7 +9,7 @@ usage()
 	# Usable?
 	which ffmpeg > /dev/null
 	mp3=0
-	[ $? -eq 0 ] && mp3=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*E.*mp3)" | grep mp3 | wc -l`
+	[ $? -eq 0 ] && mp3=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EA.*mp3)" | grep mp3 | wc -l`
 	[ "$mp3" -gt 0 ] && echo Status: Active || echo Status: Inactive
 
 	# Type
@@ -36,12 +37,12 @@ execute()
 
 	# Run the command
 	case "$profile" in 
-		"0" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 192 -ar 48000 -ac 2 -y "$file".mp3 ;;
-		"1" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 128 -ar 44100 -ac 2 -y "$file".mp3 ;;
-		"2" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 64 -ar 32000 -ac 2 -y "$file".mp3 ;;
-		"3" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 192 -ar 48000 -ac 1 -y "$file".mp3 ;;
-		"4" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 128 -ar 44100 -ac 1 -y "$file".mp3 ;;
-		"5" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 64 -ar 32000 -ac 1 -y "$file".mp3 ;;
+		"0" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 192$audio_kilo -ar 48000 -ac 2 -y "$file".mp3 ;;
+		"1" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 128$audio_kilo -ar 44100 -ac 2 -y "$file".mp3 ;;
+		"2" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 64$audio_kilo -ar 32000 -ac 2 -y "$file".mp3 ;;
+		"3" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 192$audio_kilo -ar 48000 -ac 1 -y "$file".mp3 ;;
+		"4" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 128$audio_kilo -ar 44100 -ac 1 -y "$file".mp3 ;;
+		"5" ) 	ffmpeg -f dv -i - -f mp3 -acodec mp3 -ab 64$audio_kilo -ar 32000 -ac 1 -y "$file".mp3 ;;
 	esac
 }
 
