@@ -1,4 +1,5 @@
 #!/bin/sh
+# A Kino export script that outputs to MPEG-4 part 2 MP4 using ffmpeg
 
 usage()
 {
@@ -6,17 +7,18 @@ usage()
 	echo "Title: MPEG-4 MP4 Single Pass (FFMPEG)"
 
 	# Usable?
-	aac=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*E.*aac)" | grep aac | wc -l`
+	aac=`ffmpeg -formats 2> /dev/null | egrep "(Encoders:)|(.*EA.*aac)" | grep aac | wc -l`
 	[ "$aac" -gt 0 ] && echo Status: Active || echo Status: Inactive
 
 	# Type
 	echo Flags: single-pass file-producer
 
 	# Profiles
-	echo "Profile: High Quality (640x480, VBR, QPEL)"
-	echo "Profile: Medium Quality (320x240, VBR)"
-	echo "Profile: Broadband Quality (320x240, 364 kb/s)"
-	echo "Profile: Low Quality (160x120, 12fps, 128 kb/s)"
+	echo "Profile: Best Quality (native size, interlace, VBR)"
+	echo "Profile: High Quality (full size, progressive, VBR, QPEL)"
+	echo "Profile: Medium Quality (medium size, progressive, VBR)"
+	echo "Profile: Broadband Quality (medium size, progressive, 564 kb/s)"
+	echo "Profile: Low Quality (small size, progressive, 12fps, 128 kb/s)"
 }
 
 execute()
@@ -28,8 +30,9 @@ execute()
 	file="$4"
 	project="$5"
 	pass="$6"
+	aspect="$7"
 
-	source "`dirname $0`/ffmpeg_utils.sh"
+	. "`dirname $0`/ffmpeg_utils.sh"
 	ffmpeg_generate_hq
 
 	# generate filename if missing
@@ -44,19 +47,23 @@ execute()
 	# Run the command
 	case "$profile" in 
 		"0" ) 	ffmpeg -f dv -i pipe: \
-			$hq -s 640x480 -aspect 4:3 -deinterlace -qscale 2 $qpel -ab 120 \
+			-g 300 $hq $interlace -aspect $aspect -qscale 2 -ab 192$audio_kilo \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"1" ) 	ffmpeg -f dv -i pipe: \
-			$hq -s 320x240 -aspect 4:3 -deinterlace -qscale 4 -ab 64 -ar 44100 \
+			-g 300 $hq $progressive -s $full_res -aspect $aspect -qscale 2 $qpel -ab 192$audio_kilo \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"2" ) 	ffmpeg -f dv -i pipe: \
-			$hq -s 320x240 -aspect 4:3 -deinterlace -b 300 -bt 5 -g 60 -ab 32 -ar 22050 \
+			-g 300 $hq $progressive -s $med_res -aspect $aspect -qscale 4 -ab 128$audio_kilo -ar 44100 \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 		"3" ) 	ffmpeg -f dv -i pipe: \
-			$hq -r 12 -g 36 -s 160x120 -aspect 4:3 -deinterlace -b 96 -bt 5 -ac 1 -ab 32 -ar 22050 $metadata \
+			-g 300 $hq $progressive -s $med_res -aspect $aspect -b 500$kilo -g 60 -ab 64$audio_kilo -ar 32000 \
+			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
+			-y "$file".mp4 ;;
+		"4" ) 	ffmpeg -f dv -i pipe: \
+			$hq -r 12 -g 120 $progressive -s $low_res -aspect $aspect -b 96$kilo -ac 1 -ab 32$audio_kilo -ar 22050 \
 			-title "$title" -author "$author" -comment "$comment" -copyright "$copyright" \
 			-y "$file".mp4 ;;
 	esac
