@@ -1,6 +1,7 @@
 /*
 * avi.cc library for AVI file format i/o
 * Copyright (C) 2000 - 2002 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -72,7 +73,7 @@ static char g_zeroes[ PADDING_SIZE ];
 AVIFile::AVIFile() : RIFFFile(),
 		idx1( NULL ), file_list( -1 ), riff_list( -1 ),
 		hdrl_list( -1 ), avih_chunk( -1 ), movi_list( -1 ), junk_chunk( -1 ), idx1_chunk( -1 ),
-		index_type( -1 ), current_ix00( -1 ), odml_list( -1 ), dmlh_chunk( -1 ), isUpdateIdx1( true )
+		index_type( 0 ), odml_list( -1 ), dmlh_chunk( -1 ), isUpdateIdx1( true )
 {
 	// cerr << "0x" << hex << (long)this << dec << " AVIFile::AVIFile() : RIFFFile(), ..." << endl;
 
@@ -87,6 +88,7 @@ AVIFile::AVIFile() : RIFFFile(),
 		strl_list[ i ] = -1;
 		strh_chunk[ i ] = -1;
 		strf_chunk[ i ] = -1;
+		current_ix[ i ] = -1;
 	}
 	idx1 = new AVISimpleIndex;
 	memset( idx1, 0, sizeof( AVISimpleIndex ) );
@@ -125,10 +127,10 @@ AVIFile::AVIFile( const AVIFile& avi ) : RIFFFile( avi )
 		strl_list[ i ] = avi.strl_list[ i ];
 		strh_chunk[ i ] = avi.strh_chunk[ i ];
 		strf_chunk[ i ] = avi.strf_chunk[ i ];
+		current_ix[ i ] = avi.current_ix[ i ];
 	}
 
 	index_type = avi.index_type;
-	current_ix00 = avi.current_ix00;
 
 	for ( int i = 0; i < 62; ++i )
 		dmlh[ i ] = avi.dmlh[ i ];
@@ -168,10 +170,10 @@ AVIFile& AVIFile::operator=( const AVIFile& avi )
 			strl_list[ i ] = avi.strl_list[ i ];
 			strh_chunk[ i ] = avi.strh_chunk[ i ];
 			strf_chunk[ i ] = avi.strf_chunk[ i ];
+			current_ix[ i ] = avi.current_ix[ i ];
 		}
 
 		index_type = avi.index_type;
-		current_ix00 = avi.current_ix00;
 
 		for ( int i = 0; i < 62; ++i )
 			dmlh[ i ] = avi.dmlh[ i ];
@@ -307,37 +309,32 @@ void AVIFile::Init( int format, int sampleFrequency, int indexType )
 
 int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
 {
-	switch ( index_type )
+	int result = -1;
+
+	if ( index_type & AVI_LARGE_INDEX )
 	{
-	case AVI_LARGE_INDEX:
-
-		/* find relevant index in indx0 */
-
 		int i;
 
-		for ( i = 0; frameNum >= indx[ 0 ] ->aIndex[ i ].dwDuration; frameNum -= indx[ 0 ] ->aIndex[ i ].dwDuration, ++i )
-			;
+		/* find relevant index in indx0 */
+		for ( i = 0; frameNum >= indx[ 0 ] ->aIndex[ i ].dwDuration; frameNum -= indx[ 0 ] ->aIndex[ i ].dwDuration, ++i );
 
-		if ( i != current_ix00 )
+		if ( i != current_ix[ 0 ] )
 		{
 			fail_if( lseek( fd, indx[ 0 ] ->aIndex[ i ].qwOffset + RIFF_HEADERSIZE, SEEK_SET ) == ( off_t ) - 1 );
 			fail_if( ( unsigned int )( indx[ 0 ] ->aIndex[ i ].dwSize - RIFF_HEADERSIZE ) > sizeof( AVIStdIndex ) );
 			fail_neg( read( fd, ix[ 0 ], indx[ 0 ] ->aIndex[ i ].dwSize - RIFF_HEADERSIZE ) );
-			current_ix00 = i;
+			current_ix[ 0 ] = i;
 		}
 
-		if ( frameNum < ix[ 0 ] ->nEntriesInUse )
+		if ( i < ix[ 0 ] ->nEntriesInUse )
 		{
-			fail_if( unsigned( frameNum ) >= ( sizeof( ix[ 0 ]->aIndex ) / sizeof( DWORD ) / 2 ) );
 			offset = ix[ 0 ] ->qwBaseOffset + ix[ 0 ]->aIndex[ frameNum ].dwOffset;
 			size = ix[ 0 ] ->aIndex[ frameNum ].dwSize;
-			return 0;
+			result = 0;
 		}
-		else
-			return -1;
-		break;
-
-	case AVI_SMALL_INDEX:
+	}
+	else if ( index_type & AVI_SMALL_INDEX )
+	{
 		int index = -1;
 		int frameNumIndex = 0;
 		for ( int i = 0; i < idx1->nEntriesInUse; ++i )
@@ -368,13 +365,10 @@ int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
 				offset = idx1->aIndex[ index ].dwOffset + RIFF_HEADERSIZE + GetDirectoryEntry( movi_list ).offset;
 			}
 			size = idx1->aIndex[ index ].dwSize;
-			return 0;
+			result = 0;
 		}
-		else
-			return -1;
-		break;
 	}
-	return -1;
+	return result;
 }
 
 /** Find position and size of a given frame in the file
@@ -393,31 +387,45 @@ int AVIFile::GetDVFrameInfo( off_t &offset, int &size, int frameNum )
 
 int AVIFile::GetFrameInfo( off_t &offset, int &size, int frameNum, FOURCC chunkID )
 {
+	int result = -1;
+
 	if ( index_type & AVI_LARGE_INDEX )
 	{
-		int i;
-
-		for ( i = 0; frameNum >= indx[ 0 ] ->aIndex[ i ].dwDuration; frameNum -= indx[ 0 ] ->aIndex[ i ].dwDuration, ++i )
-			;
-
-		if ( i != current_ix00 )
+		// Find the super index matches chunkID
+		for ( int j = 0; j < 2; j++ )
 		{
-			fail_if( lseek( fd, indx[ 0 ] ->aIndex[ i ].qwOffset + RIFF_HEADERSIZE, SEEK_SET ) == ( off_t ) - 1 );
-			fail_neg( read( fd, ix[ 0 ], indx[ 0 ] ->aIndex[ i ].dwSize - RIFF_HEADERSIZE ) );
-			current_ix00 = i;
-		}
-
-		if ( frameNum < ix[ 0 ] ->nEntriesInUse )
-		{
-			if ( ( FOURCC ) ix[ 0 ] ->dwChunkId == chunkID )
+			if ( indx_chunk[ j ] != -1 && indx[ j ]->dwChunkId == chunkID )
 			{
-				offset = ix[ 0 ] ->qwBaseOffset + ix[ 0 ] ->aIndex[ frameNum ].dwOffset;
-				size = ix[ 0 ] ->aIndex[ frameNum ].dwSize;
-				return 0;
+				int i = 0;
+				int totalEntries = 0;
+				int frameNumIndex = frameNum;
+
+				// Find the standard index
+				for ( i = 0; i < indx[ j ]->nEntriesInUse; ++i )
+				{
+					if ( i != current_ix[ j ] )
+					{
+						fail_if( lseek( fd, indx[ j ] ->aIndex[ i ].qwOffset + RIFF_HEADERSIZE, SEEK_SET ) == ( off_t ) - 1 );
+						fail_if( ( unsigned int )( indx[ j ] ->aIndex[ i ].dwSize - RIFF_HEADERSIZE ) > sizeof( AVIStdIndex ) );
+						fail_neg( read( fd, ix[ j ], indx[ j ] ->aIndex[ i ].dwSize - RIFF_HEADERSIZE ) );
+						current_ix[ j ] = i;
+					}
+					totalEntries += ix[ j ]->nEntriesInUse;
+					if ( frameNum <= totalEntries )
+						break;
+					frameNumIndex -= ix[ j ]->nEntriesInUse;
+				}
+				if ( i < indx[ j ]->nEntriesInUse )
+				{
+					offset = ix[ j ] ->qwBaseOffset + ix[ j ] ->aIndex[ frameNumIndex ].dwOffset;
+					size = ix[ j ] ->aIndex[ frameNumIndex ].dwSize;
+					result = 0;
+				}
+				break;
 			}
 		}
 	}
-	if ( index_type & AVI_SMALL_INDEX )
+	else if ( index_type & AVI_SMALL_INDEX )
 	{
 		int index = -1;
 		int frameNumIndex = 0;
@@ -446,10 +454,10 @@ int AVIFile::GetFrameInfo( off_t &offset, int &size, int frameNum, FOURCC chunkI
 				offset = idx1->aIndex[ index ].dwOffset + RIFF_HEADERSIZE + GetDirectoryEntry( movi_list ).offset;
 			}
 			size = idx1->aIndex[ index ].dwSize;
-			return 0;
+			result = 0;
 		}
 	}
-	return -1;
+	return result;
 }
 
 /** Read in a frame
@@ -490,8 +498,10 @@ int AVIFile::getFrame( void *data, int frameNum, FOURCC chunkID )
 
 	if ( GetFrameInfo( offset, size, frameNum, chunkID ) != 0 )
 		return 0;
+	pthread_mutex_lock( &file_mutex );
 	fail_if( lseek( fd, offset, SEEK_SET ) == ( off_t ) - 1 );
 	fail_neg( read( fd, data, size ) );
+	pthread_mutex_unlock( &file_mutex );
 
 	return size;
 }
@@ -501,6 +511,33 @@ int AVIFile::GetTotalFrames() const
 	return mainHdr.dwTotalFrames;
 }
 
+/** Determine if the audio and video frames have a 1:1 interleave factor.
+*/
+
+bool AVIFile::isInterleave1to1()
+{
+	bool result = true;
+	FOURCC dvChunkID1 = make_fourcc( "00dc" );
+	FOURCC dvChunkID2 = make_fourcc( "00db" );
+	FOURCC wavChunkID = make_fourcc( "01wb" );
+
+	// Scan the index, skipping first entry, which might be bogus (7Fxx).
+	for ( int i = 2; i < ( idx1->nEntriesInUse - 1 ); ++i )
+	{
+		// Look for DV data
+		if ( ( idx1->aIndex[ i ].dwChunkId == dvChunkID1 ||
+			   idx1->aIndex[ i ].dwChunkId == dvChunkID2 ) )
+		{
+			// Determine if WAV data is both prior and following index entries
+			if ( idx1->aIndex[ i-1 ].dwChunkId != wavChunkID || idx1->aIndex[ i+1 ].dwChunkId != wavChunkID)
+			{
+				result = false;
+				break;
+			}
+		}
+	}
+	return result;
+}
 
 /** prints out a directory entry in text form
  
@@ -802,40 +839,44 @@ void AVIFile::ParseRIFF()
 
 void AVIFile::ReadIndex()
 {
-	indx_chunk[ 0 ] = FindDirectoryEntry( make_fourcc( "indx" ) );
-	if ( indx_chunk[ 0 ] != -1 )
+	mainHdr.dwTotalFrames = 0;
+	for ( int j = 0; j < 2; j++ )
 	{
-		ReadChunk( indx_chunk[ 0 ], ( void* ) indx[ 0 ], sizeof( AVISuperIndex ) );
-		index_type = AVI_LARGE_INDEX;
-
-		/* recalc number of frames from each index */
-		mainHdr.dwTotalFrames = 0;
-		for ( int i = 0;
-		        i < indx[ 0 ] ->nEntriesInUse;
-		        mainHdr.dwTotalFrames += indx[ 0 ] ->aIndex[ i++ ].dwDuration );
-		return;
+		indx_chunk[ j ] = FindDirectoryEntry( make_fourcc( "indx" ), j );
+		if ( indx_chunk[ j ] != -1 )
+		{
+			ReadChunk( indx_chunk[ j ], ( void* ) indx[ j ], sizeof( AVISuperIndex ) );
+			index_type |= AVI_LARGE_INDEX;
+	
+			/* recalc number of frames from each index */
+			if ( j == 0 )
+			{
+				for ( int i = 0; i < indx[ j ] ->nEntriesInUse;
+					mainHdr.dwTotalFrames += indx[ j ] ->aIndex[ i++ ].dwDuration );
+			}
+		}
 	}
 	idx1_chunk = FindDirectoryEntry( make_fourcc( "idx1" ) );
 	if ( idx1_chunk != -1 )
 	{
 		ReadChunk( idx1_chunk, ( void* ) idx1, sizeof( AVISimpleIndex ) );
 		idx1->nEntriesInUse = GetDirectoryEntry( idx1_chunk ).length / 16;
-		index_type = AVI_SMALL_INDEX;
-
-		/* recalc number of frames from the simple index */
-		int frameNumIndex = 0;
-		FOURCC chunkID1 = make_fourcc( "00dc" );
-		FOURCC chunkID2 = make_fourcc( "00db" );
-		for ( int i = 0; i < idx1->nEntriesInUse; ++i )
+		index_type |= AVI_SMALL_INDEX;
+		if ( mainHdr.dwTotalFrames == 0 )
 		{
-			if ( idx1->aIndex[ i ].dwChunkId == chunkID1 ||
-			        idx1->aIndex[ i ].dwChunkId == chunkID2 )
+			/* recalc number of frames from the simple index */
+			FOURCC chunkID1 = make_fourcc( "00dc" );
+			FOURCC chunkID2 = make_fourcc( "00db" );
+
+			for ( int i = 0; i < idx1->nEntriesInUse; ++i )
 			{
-				++frameNumIndex;
+				if ( idx1->aIndex[ i ].dwChunkId == chunkID1 ||
+						idx1->aIndex[ i ].dwChunkId == chunkID2 )
+				{
+					++mainHdr.dwTotalFrames;
+				}
 			}
 		}
-		mainHdr.dwTotalFrames = frameNumIndex;
-		return ;
 	}
 }
 
@@ -1556,6 +1597,11 @@ bool AVI2File::WriteFrame( const Frame &frame )
 	audio_size = frame.ExtractAudio( soundbuf );
 	if ( audio_size > 0 )
 	{
+#if BYTE_ORDER == BIG_ENDIAN
+		int16_t* p = reinterpret_cast< int16_t* >( soundbuf );
+		for ( int i = 0; i < audio_size / sizeof( int16_t ); ++i, ++p )
+			*p = bswap_16( *p );
+#endif
 		audio_chunk = AddDirectoryEntry( make_fourcc( "01wb" ), 0, audio_size, movi_list );
 		if ( ( index_type & AVI_LARGE_INDEX ) && ( streamHdr[ 0 ].dwLength % IX00_INDEX_SIZE ) == 0 )
 		{
