@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2001-2005 Alejandro Aguilar Sierra <asierra@servidor.unam.mx>
- *  and Dan Dennedy <dan@dennedy.org>
+ * Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -61,8 +61,11 @@ DVTitler::DVTitler()
 	GtkTextBuffer* buffer = gtk_text_view_get_buffer(titleEntry);
 	g_signal_connect( G_OBJECT( buffer ), "changed", G_CALLBACK( Repaint ), 0 );
 	GtkWidget* widget = glade_xml_get_widget( glade, "fontpicker" );
+	gtk_font_button_set_font_name( GTK_FONT_BUTTON( widget ), "Sans 32" );
 	g_signal_connect( G_OBJECT( widget ), "font-set", G_CALLBACK( Repaint ), 0 );
-	widget = glade_xml_get_widget( glade, "spinpad" );
+	widget = glade_xml_get_widget( glade, "spinpadw" );
+	g_signal_connect( G_OBJECT( widget ), "value-changed", G_CALLBACK( Repaint ), 0 );
+	widget = glade_xml_get_widget( glade, "spinpadh" );
 	g_signal_connect( G_OBJECT( widget ), "value-changed", G_CALLBACK( Repaint ), 0 );
 	widget = glade_xml_get_widget( glade, "spinoutline" );
 	g_signal_connect( G_OBJECT( widget ), "value-changed", G_CALLBACK( Repaint ), 0 );
@@ -84,6 +87,8 @@ DVTitler::DVTitler()
 	g_signal_connect( G_OBJECT( widget ), "changed", G_CALLBACK( Repaint ), 0 );
 	widget = glade_xml_get_widget( glade, "optionmenufv" );
 	g_signal_connect( G_OBJECT( widget ), "changed", G_CALLBACK( Repaint ), 0 );
+	widget = glade_xml_get_widget( glade, "checkbutton_markup" );
+	g_signal_connect( G_OBJECT( widget ), "toggled", G_CALLBACK( Repaint ), 0 );
 }
 
 DVTitler::~DVTitler()
@@ -94,11 +99,6 @@ DVTitler::~DVTitler()
 	delete titler;
 }
 
-void DVTitler::Repaint( void )
-{
-	GetSelectedFramesForFX().Repaint();
-}
-	
 char *DVTitler::GetDescription( ) const
 {
 	return _("Titler");
@@ -151,7 +151,8 @@ void DVTitler::InterpretWidgets( GtkBin *bin )
 	if ( fv > 4 ) // no change
 		fv = iv;
 
-	pad = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinpad" ) ) );
+	padw = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinpadw" ) ) );
+	padh = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinpadh" ) ) );
 	int space = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinspace" ) ) );
 
 	fadein  = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinfadein" ) ) );
@@ -168,21 +169,21 @@ void DVTitler::InterpretWidgets( GtkBin *bin )
 	yoff = gtk_spin_button_get_value_as_int( GTK_SPIN_BUTTON( glade_xml_get_widget( glade, "spinyoff" ) ) );
 
 	// Obtain the text
-	g_free( text );
 	GtkTextIter start;
 	GtkTextIter end;
 	GtkTextBuffer* buffer = gtk_text_view_get_buffer( titleEntry );
 	gtk_text_buffer_get_iter_at_offset( buffer, &start, 0 );
 	gtk_text_buffer_get_iter_at_offset( buffer, &end, -1 );
+	g_free( text );
 	text = gtk_text_buffer_get_text( buffer, &start, &end, FALSE );
 
 	interlaced = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( glade_xml_get_widget( glade, "checkbutton_interlaced" ) ) );
 	isMarkup = gtk_toggle_button_get_active( GTK_TOGGLE_BUTTON( glade_xml_get_widget( glade, "checkbutton_markup" ) ) );
 
-	delete titler;
-	titler = new TextBlock();
+	if ( !titler )
+		titler = new TextBlock();
 	titler->setFont( fontname );
-	titler->setPad( pad );
+	titler->setPad( padw, padh );
 	titler->setSpace( space );
 	titler->setAlign( align );
 	titler->setText( text );
@@ -213,15 +214,8 @@ void DVTitler::FilterFrame( uint8_t *io, int width, int height, double position,
 		pixbuf = titler->getPixbuf( colorfg, colorbg, outline, coloroutline, isMarkup );
 		if ( pixbuf == 0 )
 		{
-			GtkWidget * dialog = gtk_message_dialog_new(
-		                         GetKinoWidgetWindow(),
-		                         GTK_DIALOG_DESTROY_WITH_PARENT,
-		                         GTK_MESSAGE_ERROR,
-		                         GTK_BUTTONS_CLOSE,
-		                         _("Rendering failed. Are you using bad markup?") );
-			gtk_dialog_run( GTK_DIALOG( dialog ) );
-			gtk_widget_destroy( dialog );
-			return;
+			titler->setText( _("Rendering failed.\nAre you using bad markup?") );
+			pixbuf = titler->getPixbuf( colorfg, colorbg, outline, coloroutline, false );
 		}
 
 		int w = gdk_pixbuf_get_width ( pixbuf );
@@ -402,9 +396,9 @@ extern "C"
 		switch ( index )
 		{
 		case 0:
-			return new DVTitler();
-		case 1:
 			return new Superimpose();
+		case 1:
+			return new DVTitler();
 		}
 
 		return NULL;
