@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2005 Dan Dennedy <dan@dennedy.org>
+ * Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -31,17 +31,22 @@
 #include "dvtitler.h"
 #include "../kino_extra.h"
 
+#define GAP_MAX (9999)
+
 char Superimpose::Superimpose::file[ PATH_MAX + NAME_MAX ] = "";
 
 static void
 on_button_file_clicked( GtkButton *button, gpointer user_data );
 
 Superimpose::Superimpose()
+	: hasFilenameChanged( false )
+	, count_offset( 0 )
 {
 	GtkWidget* widget = glade_xml_get_widget( glade, "button_superimpose_open" );
 	GtkWidget* widget2 = glade_xml_get_widget( glade, "entry_superimpose" );
 	g_signal_connect( G_OBJECT( widget ), "clicked", G_CALLBACK( on_button_file_clicked ), widget2 );
-	g_signal_connect( G_OBJECT( widget2 ), "changed", G_CALLBACK( Repaint ), 0 );
+	g_signal_connect( G_OBJECT( widget2 ), "activate", G_CALLBACK( Repaint ), 0 );
+	g_signal_connect( G_OBJECT( glade_xml_get_widget( glade, "hscale_superimpose_zoom" ) ), "value-changed", G_CALLBACK( Repaint ), 0 );
 }
 
 char *Superimpose::GetDescription( ) const
@@ -51,11 +56,12 @@ char *Superimpose::GetDescription( ) const
 
 void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double position, double frame_delta )
 {
+	if ( strcmp( file, "" ) == 0 )
+		return;
+
 	GError* gerror = NULL;
 	bool isUnscaled = false;
-
-	if ( !file || strcmp( file, "" ) == 0 )
-		return;
+	SelectedFrames& fx = GetSelectedFramesForFX();
 	
 	// Determine if this is an image sequence
 	if ( strchr( file, '%' ) != NULL )
@@ -63,26 +69,44 @@ void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double po
 		char full[ PATH_MAX + NAME_MAX ];
 		int gap = 0;
 	
+		if ( fx.IsRepainting() || fx.IsPreviewing() )
+		{
+			// compute the relative count
+			count = fx.GetIndex( position );
+			
+			// determine offset to first file
+			if ( hasFilenameChanged )
+			for ( count_offset = 0; count_offset < GAP_MAX; ++count_offset )
+			{
+				struct stat buf;
+				snprintf( full, PATH_MAX + NAME_MAX, file, count + count_offset );
+				if ( stat( full, &buf ) == 0 )
+					break;
+			}
+			count += count_offset;
+		}
 		// Permit gaps in the file number sequence up to 100 difference
-		while ( gap < 100 )
+		while ( gap < GAP_MAX )
 		{
 			struct stat buf;
-				
 			snprintf( full, PATH_MAX + NAME_MAX, file, count++ );
 			if ( stat( full, &buf ) == 0 )
 				break;
 			else
 				gap ++;
 		}
-		
 		// If successfully obtained a new file
-		if ( gap < 100 )
+		if ( gap < GAP_MAX )
+		{
 			pixbuf = gdk_pixbuf_new_from_file( full, &gerror );
-		isUnscaled = true;
+			isUnscaled = true;
+		}
 	}
 	// Otherwise, import single image just once
-	else if ( position == 0 || GetSelectedFramesForFX().IsRepainting() )
+	else if ( hasFilenameChanged || fx.IsRepainting() )
 	{
+		if ( pixbuf )
+			g_object_unref( pixbuf );
 		pixbuf = gdk_pixbuf_new_from_file( file, &gerror );
 		isUnscaled = true;
 	}
@@ -90,19 +114,28 @@ void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double po
 	// If gdk-pixbuf loader generated an error
 	if ( gerror != NULL )
 	{
-		GtkWidget * dialog = gtk_message_dialog_new(
-								GetKinoWidgetWindow(),
-								GTK_DIALOG_DESTROY_WITH_PARENT,
-								GTK_MESSAGE_ERROR,
-								GTK_BUTTONS_CLOSE,
-								gerror->message );
-		gtk_dialog_run( GTK_DIALOG( dialog ) );
-		gtk_widget_destroy( dialog );
+		if ( hasFilenameChanged )
+		{
+			if ( fx.IsPreviewing() )
+				gdk_threads_enter();
+			GtkWidget * dialog = gtk_message_dialog_new(
+									GetKinoWidgetWindow(),
+									GTK_DIALOG_DESTROY_WITH_PARENT,
+									GTK_MESSAGE_ERROR,
+									GTK_BUTTONS_CLOSE,
+									gerror->message );
+			gtk_dialog_run( GTK_DIALOG( dialog ) );
+			gtk_widget_destroy( dialog );
+			if ( fx.IsPreviewing() )
+				gdk_threads_leave();
+		}
 		g_error_free( gerror );
-		return;
+		if ( pixbuf )
+			g_object_unref( pixbuf );
+		pixbuf = 0;
 	}
 	// Otherwise, if there is a pixbuf
-	else if ( isUnscaled )
+	else if ( pixbuf && isUnscaled )
 	{
 		if ( gdk_pixbuf_get_has_alpha( pixbuf ) == FALSE )
 		{
@@ -130,7 +163,7 @@ void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double po
 		{
 			GdkPixbuf * temp = pixbuf;
 			GdkPixbuf *scaled = gdk_pixbuf_scale_simple( pixbuf,
-				gint( float( w ) * 720.0/640.0 ), h, interp );
+				gint( double( w ) * 720.0/640.0 * zoom + 0.5 ), gint( h * zoom + 0.5 ), interp );
 			pixbuf = scaled;
 			g_object_unref( temp );
 		}
@@ -139,7 +172,7 @@ void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double po
 		{
 			GdkPixbuf * temp = pixbuf;
 			GdkPixbuf *scaled = gdk_pixbuf_scale_simple( pixbuf,
-				gint( float( w ) * 720.0/768.0 ), h, interp );
+				gint( float( w ) * 720.0/768.0 * zoom + 0.5 ), gint( h * zoom + 0.5 ), interp );
 			pixbuf = scaled;
 			g_object_unref( temp );
 		}
@@ -186,11 +219,13 @@ void Superimpose::FilterFrame( uint8_t *pixels, int width, int height, double po
                                     position, frame_delta );
 		}
 	}
+	hasFilenameChanged = false;
 }
 
 static void
 on_button_file_clicked( GtkButton *button, gpointer user_data )
 {
+	GtkWidget *widget = static_cast< GtkWidget* >( user_data );
 	GtkWidget *dialog;
 	gchar *filename = NULL;
 
@@ -202,14 +237,16 @@ on_button_file_clicked( GtkButton *button, gpointer user_data )
 				NULL );
 	gtk_dialog_set_alternative_button_order( GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1 );
 	gtk_window_set_modal( GTK_WINDOW( dialog ), TRUE );
+	char* folder = strdup( gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+	if ( strrchr( folder, '/' ) )
+		*( strrchr( folder, '/' ) + 1 ) = '\0';
+	gtk_file_chooser_set_current_folder( GTK_FILE_CHOOSER( dialog ), folder );
+	free( folder );
 	if ( gtk_dialog_run( GTK_DIALOG( dialog ) ) == GTK_RESPONSE_ACCEPT )
 		filename = gtk_file_chooser_get_filename( GTK_FILE_CHOOSER( dialog ) );
 	gtk_widget_destroy( dialog );
 	if ( filename && strcmp( filename, "" ) )
-	{
-		GtkWidget *widget = static_cast< GtkWidget* >( user_data );
 		gtk_entry_set_text( GTK_ENTRY( widget ), filename );
-	}
 	g_free( filename );
 }
 	
@@ -270,10 +307,23 @@ void Superimpose::InterpretWidgets( GtkBin *bin )
 
 	widget = glade_xml_get_widget( glade, "entry_superimpose" );
 	if ( gtk_entry_get_text( GTK_ENTRY( widget ) ) )
-		strcpy( file, gtk_entry_get_text( GTK_ENTRY( widget ) ) );
+	{
+		int n = PATH_MAX + NAME_MAX - 1;
+		char newFileName[ n + 1 ];
+		newFileName[ n ] = '\0';
+		strncpy( newFileName, gtk_entry_get_text( GTK_ENTRY( widget ) ), n );
+		SelectedFrames& fx = GetSelectedFramesForFX();
+		if ( strcmp( newFileName, file ) || !( fx.IsRepainting() || fx.IsPreviewing() ) )
+			hasFilenameChanged = true;
+		strcpy( file, newFileName );
+	}
 	else
+	{
 		throw _( "No image file name specified - aborting." );
+	}
 	count = 0;
+	widget = glade_xml_get_widget( glade, "hscale_superimpose_zoom" );
+	zoom = gtk_range_get_value( GTK_RANGE( widget ) ) / 100.0;
 	DVTitler::InterpretWidgets( bin );
 }
 
