@@ -1,6 +1,7 @@
 /*
 * ieee1394io.cc -- asynchronously grabbing DV data
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -708,7 +709,7 @@ bool dv1394Reader::Handler( int handle )
 
 	if ( dvst.dropped_frames > 0 )
 	{
-		printf( "dv1394 reported %d dropped frames.\n", dvst.dropped_frames );
+		std:cerr << "dv1394 reported " << dvst.dropped_frames << " dropped frames." << std::endl;
 		droppedFrames += dvst.dropped_frames;
 	}
 
@@ -763,7 +764,7 @@ bool dv1394Reader::Handler( int handle )
 
 		if ( dvst.dropped_frames > 0 )
 		{
-			printf( "dv1394 reported %d dropped frames.\n", dvst.dropped_frames );
+			std::cerr << "dv1394 reported " << dvst.dropped_frames << " dropped frames." << std::endl;
 			droppedFrames += dvst.dropped_frames;
 		}
 
@@ -1142,8 +1143,10 @@ bool AVC::Timecode( int phyID, char* timecode )
 				pthread_mutex_unlock( &avc_mutex );
 				return false;
 			}
-
-			sprintf( timecode, "%2.2x:%2.2x:%2.2x:%2.2x",
+			if ( response[1] == 0xffffffff )
+				strcpy( timecode, "--:--:--:--" );
+			else
+				sprintf( timecode, "%2.2x:%2.2x:%2.2x:%2.2x",
 			         response[ 1 ] & 0x000000ff,
 			         ( response[ 1 ] >> 8 ) & 0x000000ff,
 			         ( response[ 1 ] >> 16 ) & 0x000000ff,
@@ -1241,39 +1244,48 @@ void IEEE1394Writer::StopThread()
 	}
 }
 
-unsigned int IEEE1394Writer::WaitForAction( int seconds )
+int IEEE1394Writer::WaitForAction( bool isBlocking, int seconds )
 {
 	pthread_mutex_lock( &m_dequeMutex );
-	unsigned int size = m_deque.size( );
+	int size = m_deque.size( );
 	pthread_mutex_unlock( &m_dequeMutex );
 
-	if ( size >= m_nBuffers )
+	if ( ( unsigned int )size >= m_nBuffers )
 	{
-		pthread_mutex_lock( &m_conditionMutex );
-		if ( seconds == 0 )
+		if ( isBlocking )
 		{
-			pthread_cond_wait( &m_condition, &m_conditionMutex );
-			pthread_mutex_unlock( &m_conditionMutex );
-			pthread_mutex_lock( &m_dequeMutex );
-			size = m_deque.size( );
+			pthread_mutex_lock( &m_conditionMutex );
+			if ( seconds == 0 )
+			{
+				pthread_cond_wait( &m_condition, &m_conditionMutex );
+				pthread_mutex_unlock( &m_conditionMutex );
+				pthread_mutex_lock( &m_dequeMutex );
+				size = m_deque.size( );
+			}
+			else
+			{
+				struct timeval tp;
+				struct timespec ts;
+				int result;
+	
+				gettimeofday( &tp, NULL );
+				ts.tv_sec = tp.tv_sec + seconds;
+				ts.tv_nsec = tp.tv_usec * 1000;
+	
+				result = pthread_cond_timedwait( &m_condition, &m_conditionMutex, &ts );
+				pthread_mutex_unlock( &m_conditionMutex );
+				pthread_mutex_lock( &m_dequeMutex );
+	
+				size = m_deque.size();
+				if ( ( unsigned int )size >= m_nBuffers )
+					size = -size;
+			}
+			pthread_mutex_unlock( &m_dequeMutex );
 		}
 		else
 		{
-			struct timeval tp;
-			struct timespec ts;
-			int result;
-
-			gettimeofday( &tp, NULL );
-			ts.tv_sec = tp.tv_sec + seconds;
-			ts.tv_nsec = tp.tv_usec * 1000;
-
-			result = pthread_cond_timedwait( &m_condition, &m_conditionMutex, &ts );
-			pthread_mutex_unlock( &m_conditionMutex );
-			pthread_mutex_lock( &m_dequeMutex );
-
-			size = m_deque.size();
+			size = -size;
 		}
-		pthread_mutex_unlock( &m_dequeMutex );
 	}
 
 	return size;
@@ -1410,14 +1422,16 @@ bool iec61883Writer::Open( bool isPAL )
 		raw1394_set_bus_reset_handler( m_handle, this->ResetHandlerProxy );
 		m_iec61883dv = iec61883_dv_xmit_init( m_handle, isPAL, HandlerProxy, this );
 		m_isInitialised = ( m_iec61883dv != NULL );
-		iec61883_dv_set_synch( m_iec61883dv, 1 );
+		//iec61883_dv_set_synch( m_iec61883dv, 1 ); this is causing hang on close
 	}
 
 	return m_isInitialised;
 }
 
-void iec61883Writer::SendFrame( Frame &frame )
+bool iec61883Writer::SendFrame( Frame &frame, bool isBlocking )
 {
+	bool result = false;
+
 	if ( m_isReset )
 	{
 		StopThread();
@@ -1433,21 +1447,24 @@ void iec61883Writer::SendFrame( Frame &frame )
 	}
 
 	// Block if too full
-	WaitForAction();
-
-	// Duplicate the frame
-	Frame *newFrame = GetFramePool()->GetFrame();
-	*newFrame = frame;
-
-	pthread_mutex_lock( &m_dequeMutex );
-	m_deque.push_back( newFrame );
-	pthread_mutex_unlock( &m_dequeMutex );
-
-	if ( !m_isInitialised && Open( frame.IsPAL() ) )
+	if ( WaitForAction( isBlocking ) >= 0 )
 	{
-		m_isInitialised = ( iec61883_dv_xmit_start( m_iec61883dv, m_channel ) == 0 );
-		StartThread();
+		// Duplicate the frame
+		Frame *newFrame = GetFramePool()->GetFrame();
+		*newFrame = frame;
+	
+		pthread_mutex_lock( &m_dequeMutex );
+		m_deque.push_back( newFrame );
+		pthread_mutex_unlock( &m_dequeMutex );
+	
+		if ( !m_isInitialised && Open( frame.IsPAL() ) )
+		{
+			m_isInitialised = ( iec61883_dv_xmit_start( m_iec61883dv, m_channel ) == 0 );
+			StartThread();
+		}
+		result = true;
 	}
+	return result;
 }
 
 void* iec61883Writer::Thread()
@@ -1512,41 +1529,46 @@ dv1394Writer::~dv1394Writer()
 }
 
 
-void dv1394Writer::SendFrame( Frame &frame )
+bool dv1394Writer::SendFrame( Frame &frame, bool isBlocking )
 {
+	bool result = false;
+
 	if ( m_fd < 0 )
-		return;
+		return result;
 
 	bool isPAL = frame.IsPAL();
 
 	// Block if too full
-	WaitForAction();
-
-	// Duplicate the frame
-	Frame *newFrame = GetFramePool()->GetFrame();
-	*newFrame = frame;
-
-	pthread_mutex_lock( &m_dequeMutex );
-	m_deque.push_back( newFrame );
-	pthread_mutex_unlock( &m_dequeMutex );
-
-	if ( !m_isInitialised )
+	if ( WaitForAction( isBlocking ) >= 0 )
 	{
-		m_isInitialised = true;
-
-		struct dv1394_init setup =
-		    {
-			api_version: DV1394_API_VERSION,
-			channel: m_channel,
-			n_frames: m_nBuffers,
-			format: ( isPAL ? DV1394_PAL : DV1394_NTSC ),
-			cip_n : m_cip_n,
-			cip_d : m_cip_d,
-			syt_offset : m_syt_offset
-		    };
-		ioctl( m_fd, DV1394_INIT, &setup );
-		StartThread();
+		// Duplicate the frame
+		Frame *newFrame = GetFramePool()->GetFrame();
+		*newFrame = frame;
+	
+		pthread_mutex_lock( &m_dequeMutex );
+		m_deque.push_back( newFrame );
+		pthread_mutex_unlock( &m_dequeMutex );
+	
+		if ( !m_isInitialised )
+		{
+			m_isInitialised = true;
+	
+			struct dv1394_init setup =
+				{
+				api_version: DV1394_API_VERSION,
+				channel: m_channel,
+				n_frames: m_nBuffers,
+				format: ( isPAL ? DV1394_PAL : DV1394_NTSC ),
+				cip_n : m_cip_n,
+				cip_d : m_cip_d,
+				syt_offset : m_syt_offset
+				};
+			ioctl( m_fd, DV1394_INIT, &setup );
+			StartThread();
+		}
+		result = true;
 	}
+	return result;
 }
 
 void* dv1394Writer::Thread()
