@@ -1,6 +1,7 @@
 /*
 * kino_common.cc KINO GUI Common Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -31,7 +32,9 @@ using std::endl;
 #include <sys/types.h>
 #include <sys/wait.h>
 #include <dirent.h>
-#include <libgen.h>
+#ifndef _GNU_SOURCE
+#define _GNU_SOURCE
+#endif
 #include <string.h>
 #include <stdarg.h>
 #include <time.h>
@@ -60,6 +63,7 @@ using std::endl;
 #include "filehandler.h"
 #include "ieee1394io.h"
 #include "error.h"
+#include "stringutils.h"
 
 
 extern "C"
@@ -378,12 +382,19 @@ bool KinoCommon::newFile( bool prompt )
 	// added for tagesschau.de
 	if ( prompt && strcmp( Preferences::getInstance().newProjectURI, "" ) )
 	{
-		char *response = modal_prompt( _("Enter a project name") );
-		if ( response )
+		if ( strstr( Preferences::getInstance().newProjectURI, "%" ) )
 		{
-			getPlayList()->SetDocTitle( response );
-			fetchProjectMetadata( response );
-			free( response );
+			char *response = modal_prompt( _("Enter a project name") );
+			if ( response )
+			{
+				getPlayList()->SetDocTitle( response );
+				fetchProjectMetadata( response );
+				free( response );
+			}
+		}
+		else
+		{
+			fetchProjectMetadata( "_" );
 		}
 	}
 	return result;
@@ -631,7 +642,7 @@ std::string KinoCommon::importFile( const char *filename )
 			args[ 5 ] = const_cast<char*>( strstream.str().c_str() );
 			GetFramePool()->DoneWithFrame( &frame );
 		}
-		importedFile += ".dv";
+		importedFile = StringUtils::replaceAll( importedFile, "\"", "\\\"") + std::string( ".dv" );
 		
 		args[ 0 ] = DATADIR "/kino/scripts/import/media.sh";
 		args[ 1 ] = const_cast<char*>( filename );
@@ -660,6 +671,18 @@ std::string KinoCommon::importFile( const char *filename )
  			g_object_get( G_OBJECT( dialog ), "visible", &visible, static_cast<gchar*>( NULL ) );
 			waitpid( pid, &status, WNOHANG );
 		}
+		if ( ! WIFEXITED( status ) )
+		{
+			kill( pid, SIGTERM );
+			waitpid( pid, &status, WNOHANG );
+			if ( ! WIFEXITED( status ) )
+			{
+ 				timespec ts = { 0, 100000000L };
+				nanosleep( &ts, NULL );
+				kill( pid, SIGKILL );
+				waitpid( pid, &status, 0 );
+			}
+		}
 		gtk_widget_hide( dialog );
 	}
 	return importedFile;
@@ -849,10 +872,30 @@ bool KinoCommon::savePlayListAs( )
 {
 	bool result = false;
 	int format = 1;
-	char *filename = this->getFileToSaveFormat( _( "Save the movie as SMIL" ), format );
+	char *filename = this->getFileToSaveFormat( _( "Save the movie" ), format );
 
 	if ( filename && strcmp( filename, "" ) )
 	{
+		// Append an extension if not supplied
+		char* extension = strrchr( filename, '.' );
+		switch ( format )
+		{
+			case 0: // SMIL
+			case 1: // Legacy XML
+				if ( !extension || ( strcmp( extension, ".kino" ) && 
+					strcmp( extension, ".smil" ) && strcmp( extension, ".xml" ) ) )
+					strcat( filename, ".kino" );
+				break;
+			case 2: // ELI
+				if ( !extension || strcmp( extension, ".eli" ) )
+					strcat( filename, ".eli" );
+				break;
+			case 3: // SRT - SubRip subtitle
+				if ( !extension || strcmp( extension, ".srt" ) )
+					strcat( filename, ".srt" );
+				break;
+		}
+		
 		// See if it exists already and warn user
 		FILE *f = fopen( filename, "r" );
 		if ( f )
@@ -892,6 +935,13 @@ bool KinoCommon::savePlayListAs( )
 					result = false;
 				}
 				break;
+			case 3: // SRT - SubRip subtitle
+				if ( !this->getPlayList()->SavePlayListSrt( filename ) )
+				{
+					modal_message( _( "Could not save the subtitle" ) );
+					result = false;
+				}
+				break;
 		}
 	}
 	return result;
@@ -929,7 +979,6 @@ void KinoCommon::saveFrame( )
 	}
 }
 
-#ifdef HAVE_GTK24
 /**
 
 Used by update_preview_cb to generate a file preview in the File->Open dialog
@@ -970,6 +1019,7 @@ gboolean generate_file_preview( GtkWidget *preview, const char *filename )
 				GdkDisplayer* display = new GdkDisplayer( preview, frame.GetWidth(), frame.GetHeight(), frame.IsWide() );
 				unsigned char *pixels = new unsigned char[ FRAME_MAX_WIDTH * FRAME_MAX_HEIGHT * 4 ];
 				frame.ExtractPreviewRGB( pixels );
+				gdk_window_clear( preview->window );
 				display->put( pixels );
 				delete[] pixels;
 				delete display;
@@ -1116,6 +1166,7 @@ char *KinoCommon::getFileToSaveFormat( char *title, GtkWidget *widget, int& form
 					static_cast<gchar*>( NULL ) );
 	gtk_dialog_set_alternative_button_order (GTK_DIALOG (dialog), GTK_RESPONSE_ACCEPT, GTK_RESPONSE_CANCEL, -1);
 	gtk_window_set_modal( GTK_WINDOW( dialog ), TRUE );
+	gtk_dialog_set_default_response( GTK_DIALOG( dialog ), GTK_RESPONSE_ACCEPT );
 
 	if ( format )
 	{
@@ -1130,6 +1181,7 @@ char *KinoCommon::getFileToSaveFormat( char *title, GtkWidget *widget, int& form
 		gtk_combo_box_append_text( formatCombo, "SMIL 2.0" );
 		gtk_combo_box_append_text( formatCombo, "Kino < 0.9.1 XML" );
 		gtk_combo_box_append_text( formatCombo, "MJPEG Tools ELI" );
+		gtk_combo_box_append_text( formatCombo, "SRT subtitle" );
 		gtk_combo_box_set_active( formatCombo, 0 );
 		gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 0 );
 		gtk_box_pack_start( GTK_BOX(hbox), GTK_WIDGET(formatCombo), FALSE, FALSE, 0 );
@@ -1165,52 +1217,6 @@ char *KinoCommon::getFileToSaveFormat( char *title, GtkWidget *widget, int& form
 	return tempFileName;
 }
 
-#else /* HAVE_GTK24 */
-
-char *KinoCommon::getFileToOpen( char *title, bool isDVFile )
-{
-	GtkWidget *dialog = gtk_file_selection_new( title );
-	gtk_window_set_modal( GTK_WINDOW( dialog ), TRUE );
-	g_signal_connect( G_OBJECT( GTK_FILE_SELECTION( dialog )->ok_button ),
-				"clicked", G_CALLBACK( gtk_widget_hide ),
-				(gpointer) dialog );
-	g_signal_connect( G_OBJECT( GTK_FILE_SELECTION( dialog )->cancel_button ),
-				"clicked", G_CALLBACK( gtk_widget_hide ),
-				(gpointer) dialog );
-	if ( last_directory == "" )
-	{
-		string directory = getPlayList() ->GetProjectDirectory( );
-		if ( directory != "" )
-			gtk_file_selection_set_filename( GTK_FILE_SELECTION( dialog ), ( char * ) ( directory + "/" ).c_str() );
-		last_directory = directory;
-	}
-	else
-	{
-		gtk_file_selection_set_filename( GTK_FILE_SELECTION( dialog ), ( char * ) ( last_directory + "/" ).c_str() );
-	}
-	gtk_dialog_run( GTK_DIALOG( dialog ) );
-	const gchar *filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION( dialog ) );
-	/* If the user cancels, a directory will still be returned
-	   If this is the case, return "" - the caller knows how to handle this */
-	if ( opendir( filename ) == NULL )
-	{
-		strcpy( tempFileName, filename );
-		last_directory = directory_utils::get_directory_from_file( filename );
-	}
-	else
-		strcpy( tempFileName, "" );
-	g_free( filename );
-	gtk_widget_destroy( dialog );
-	return tempFileName;
-}
-
-char *KinoCommon::getFileToSave( char *title )
-{
-	return getFileToOpen(title);
-}
-
-#endif
-
 /**	Determines the format of the file.
 
 	It is required that a file be at least 20 bytes for it to load.
@@ -1296,6 +1302,11 @@ void KinoCommon::setWindowTitle( )
 		strcat( strbuf, " - Kino" );
 		gtk_window_set_title( GTK_WINDOW( mainWindow ), strbuf );
 	}
+	if ( getPlayList()->GetDocName( ) != "" && !getPlayList()->IsDirty() )
+	{
+		Preferences::getInstance().addRecentFile( getPlayList()->GetDocName() );
+		updateRecentFiles();
+	}
 }
 
 /** Loads the specified media file before the specified frame.
@@ -1433,16 +1444,38 @@ void KinoCommon::bulkLoad( int argc, char* argv[] )
 				if ( currentPage == PAGE_TRIM )
 				{
 					gtk_entry_set_text( GTK_ENTRY( lookup_widget( widget, "entry_trim_clip" ) ), filename );
+					break;
 				}
 				else if ( loadMediaObject( filename, this->getPlayList() ->GetNumFrames() ) )
 				{
 					if ( g_currentFrame == -1 )
 						g_currentFrame = 0;
 					this->hasListChanged = TRUE;
+					break;
 				}
 				else
 				{
 					cerr << "KinoCommon::bulkLoad: Failed to load " << filename << endl;
+					// Else, ask to import
+				}
+			case UNKNOWN_FORMAT:
+				{
+					const std::string& importedFile = importFile( filename );
+					if ( currentPage == PAGE_TRIM )
+					{
+						gtk_entry_set_text( GTK_ENTRY( lookup_widget( widget, "entry_trim_clip" ) ), importedFile.c_str() );
+					}
+					else if ( loadMediaObject( const_cast<char*>( importedFile.c_str() ), this->getPlayList() ->GetNumFrames() ) )
+					{
+						if ( g_currentFrame == -1 )
+							g_currentFrame = 0;
+						hasListChanged = TRUE;
+						getPlayList() ->SetDirty( true );
+					}
+					else
+					{
+						modal_message( _( "Failed to load media file \"%s\"" ), importedFile.c_str() );
+					}
 				}
 				break;
 			case PLAYLIST:
@@ -1467,14 +1500,6 @@ void KinoCommon::bulkLoad( int argc, char* argv[] )
 					}
 				}
 				break;
-			case UNKNOWN_FORMAT:
-				/* DRD> I had to comment out the following to prevent lockups with
-				  invalid args. When Xv crashes my desktop, and I log back in, Kino is
-				  started automatically by GNOME which passes some weird --smclient arg */ 
-				//				modal_message( "Invalid file specified." );
-				cerr << "KinoCommon::bulkLoad: Invalid file specified: "
-				<< filename << endl;
-				break;
 			}
 		}
 		else
@@ -1512,6 +1537,8 @@ void KinoCommon::saveFrame( int abs_frame_num, char *file )
 			int width = frame->GetWidth();
 			if ( frame->IsWide() )
 				width = frame->IsPAL() ? 1024 : 854;
+			else
+				width = frame->IsPAL() ? 768 : 640;
 			AspectRatioCalculator calc( width, frame->GetHeight(),
 		                            frame->GetWidth(), frame->GetHeight(),
 		                            frame->IsPAL(), frame->IsWide() );
@@ -1586,14 +1613,9 @@ int KinoCommon::moveToFrame( )
 
 int KinoCommon::moveToFrame( int frame )
 {
-	if ( currentPage == PAGE_TRIM )
+	if ( currentPage == PAGE_TRIM || currentPage == PAGE_MAGICK )
 	{
-		getPageTrim() ->movedToFrame( frame );
-		return frame;
-	}
-	if ( currentPage == PAGE_MAGICK )
-	{
-		getPageMagick() ->movedToFrame( frame );
+		getCurrentPage()->movedToFrame( frame );
 		return frame;
 	}
 
@@ -1678,8 +1700,8 @@ void KinoCommon::showFrameMoreInfo( Frame &frame, FileHandler *media )
 				format = "unknown";
 
 
-			snprintf( s, 2048, _( "File Name:   %s\nDV Recording Date: %s\nDV Timecode: %2.2d:%2.2d:%2.2d:%2.2d\nFile Format: %s\nAudio: %d bit, %d KHz, %d samples\nVideo: %d x %d, %s, %s, %2.2f fps" ),
-			          media->GetFilename().c_str(),
+			snprintf( s, 2048, _( "%s\n%s\n%2.2d:%2.2d:%2.2d:%2.2d\n%s\n%d bit, %d KHz, %d samples\n%d x %d, %s, %s, %2.2f fps" ),
+			          basename( media->GetFilename().c_str() ),
 			          frame.GetRecordingDate().c_str(),
 			          tc.hour, tc.min, tc.sec, tc.frame,
 			          format.c_str(),
@@ -2129,7 +2151,6 @@ void KinoCommon::setPreviewSize( float factor, bool noWarning )
 		Frame *frame = getPageCapture()->getFrame();
 		if ( frame != NULL )
 		{
-			width = frame->GetWidth();
 			if ( frame->IsWide() )
 				width = frame->IsPAL() ? 1024 : 854;
 			AspectRatioCalculator calc( width, frame->GetHeight(),
@@ -2144,9 +2165,10 @@ void KinoCommon::setPreviewSize( float factor, bool noWarning )
 	{
 		Frame& frame = *GetFramePool()->GetFrame();
 		this->getPlayList() ->GetFrame( this->g_currentFrame, frame );
-		width = frame.GetWidth();
 		if ( frame.IsWide() )
 			width = frame.IsPAL() ? 1024 : 854;
+		else
+			width = frame.IsPAL() ? 768 : 640;
 		AspectRatioCalculator calc( width, frame.GetHeight(),
 		                            frame.GetWidth(), frame.GetHeight(),
 		                            frame.IsPAL(), frame.IsWide() );
@@ -2160,7 +2182,8 @@ void KinoCommon::setPreviewSize( float factor, bool noWarning )
 		height = Preferences::getInstance().defaultNormalisation == NORM_PAL ? 576 : 480;
 		isPAL = Preferences::getInstance().defaultNormalisation == NORM_PAL;
 		isWide = Preferences::getInstance().defaultAspect == ASPECT_169;
-		AspectRatioCalculator calc( width, height, width, height,
+		AspectRatioCalculator calc( isWide ? (isPAL ? 1024 : 854) : (isPAL ? 768 : 640),
+		                            height, width, height,
 		                            isPAL, isWide );
 
 		width = ( int ) ( calc.width * factor );
@@ -2322,21 +2345,31 @@ void KinoCommon::publishPlayList()
 		mkstemp( filename );
 		copy->SavePlayList( filename );
 		delete copy;
-		command << DATADIR << "/kino/scripts/publish/project.sh ";
+		command << "\"" << DATADIR << "/kino/scripts/publish/project.sh\" ";
 		command << "\"" << filename << "\" ";
 		command << "\"" << playlist.GetDocId().c_str() << "\" ";
 		command << "\"" << playlist.GetDocTitle().c_str() << "\" ";
-		g_spawn_command_line_sync( command.str().c_str(), NULL, NULL, NULL, &gerror );
+		char * args[ 4 ];
+		args[ 0 ] = "/bin/sh";
+		args[ 1 ] = "-c";
+		args[ 2 ] = const_cast< char* >( command.str().c_str() );
+		args[ 3 ] = NULL;
+		g_spawn_sync( ".", args, NULL, (GSpawnFlags)0, NULL, NULL, NULL, NULL, NULL, &gerror );
 		unlink( filename );
 	}
 	else
 	{
 		savePlayList();
-		command << DATADIR << "/kino/scripts/publish/project.sh ";
+		command << "\"" << DATADIR << "/kino/scripts/publish/project.sh\" ";
 		command << "\"" << playlist.GetDocName() << "\" ";
 		command << "\"" << playlist.GetDocId().c_str() << "\" ";
 		command << "\"" << playlist.GetDocTitle().c_str() << "\" ";
-		g_spawn_command_line_sync( command.str().c_str(), NULL, NULL, NULL, &gerror );
+		char * args[ 4 ];
+		args[ 0 ] = "/bin/sh";
+		args[ 1 ] = "-c";
+		args[ 2 ] = const_cast< char* >( command.str().c_str() );
+		args[ 3 ] = NULL;
+		g_spawn_async_with_pipes( ".", args, NULL, (GSpawnFlags)0, NULL, NULL, NULL, NULL, NULL, NULL, &gerror );
 	}
 }
 
@@ -2356,13 +2389,80 @@ void KinoCommon::publishFrame()
 			
 			common->saveFrame( common->g_currentFrame, filename );
 			
-			command << DATADIR << "/kino/scripts/publish/frame.sh ";
+			command << "\"" << DATADIR << "/kino/scripts/publish/frame.sh\" ";
 			command << "\"" << filename << "\" ";
 			command << "\"" << getPlayList()->GetDocId().c_str() << "\" ";
 			command << "\"" << getPlayList()->GetSeqAttribute( g_currentFrame, "title" ) << "\" ";
 			command << "\"" << getPlayList()->GetDocTitle().c_str() << "\" ";
-			g_spawn_command_line_async( command.str().c_str(), &gerror );
+			char * args[ 4 ];
+			args[ 0 ] = "/bin/sh";
+			args[ 1 ] = "-c";
+			args[ 2 ] = const_cast< char* >( command.str().c_str() );
+			args[ 3 ] = NULL;
+			g_spawn_async_with_pipes( ".", args, NULL, (GSpawnFlags)0, NULL, NULL, NULL, NULL, NULL, NULL, &gerror );
 		}
 	}
 	
 }
+
+void KinoCommon::loadPlayList( char* filename )
+{
+	if ( newFile( false ) )
+	{
+		GetEditorBackup() ->Clear();
+		if ( loadPlayList( filename, 0 ) )
+		{
+			g_currentFrame = 0;
+			hasListChanged = TRUE;
+			getPageEditor( ) ->snapshot( );
+			getPlayList( ) ->SetDocName( filename );
+			getPlayList( ) ->SetDirty( false );
+			fetchProjectMetadata( getPlayList()->GetDocId() );
+		}
+	}
+	setWindowTitle();
+	this->currentScene = 0;
+	this->getCurrentPage()->clean();
+	this->getCurrentPage()->start();
+	moveToFrame( );
+	setLastDirectory( directory_utils::get_directory_from_file( filename ) );
+}
+
+
+static void on_open_recent_activate ( GtkWidget *menuitem, gpointer user_data )
+{
+	common->loadPlayList( static_cast< char* >( user_data ) );
+}
+
+void KinoCommon::updateRecentFiles()
+{
+	GtkWidget* menu = lookup_widget( widget, "file_menu" );
+	vector< std::string >& recentFiles = Preferences::getInstance().recentFiles;
+	int count = recentFiles.size();
+	const int menuPosition = Preferences::getInstance().enablePublish ? 12 : 10;
+
+	if ( recentMenuItems.empty() && !recentFiles.empty() )
+	{
+		GtkWidget* widget = gtk_separator_menu_item_new();
+		gtk_menu_shell_insert( GTK_MENU_SHELL( menu ), widget, menuPosition );
+		gtk_widget_show( widget );
+	}
+	for ( vector< GtkWidget* >::iterator i = recentMenuItems.begin(); i != recentMenuItems.end(); i++ )
+	{
+		gtk_container_remove( GTK_CONTAINER( menu ), *i );
+		gtk_widget_destroy( *i );
+	}
+	recentMenuItems.clear();
+	for ( vector< std::string >::iterator i = recentFiles.begin(); i != recentFiles.end(); i++, count-- )
+	{
+		std::ostringstream ss;
+		ss << "_" << count << ". "
+			<< StringUtils::replaceAll( basename( ( *i ).c_str() ), "_", "__" )
+			<< std::ends;
+		GtkWidget* widget = gtk_menu_item_new_with_mnemonic( ss.str().c_str() );
+		g_signal_connect( G_OBJECT( widget ), "activate", G_CALLBACK( on_open_recent_activate ), gpointer( (*i).c_str() ) );
+		gtk_menu_shell_insert( GTK_MENU_SHELL( menu ), widget, menuPosition );
+		gtk_widget_show( widget );
+		recentMenuItems.push_back( widget );
+	}
+}
