@@ -1,6 +1,7 @@
 /*
 * kino_common.h KINO GUI Common Object
-* Copyright (C) 2000 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -20,6 +21,8 @@
 #ifndef _KINO_COMMON_H
 #define _KINO_COMMON_H
 
+#include <vector>
+
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
 #include <gdk/gdkkeysyms.h>
@@ -154,6 +157,7 @@ private:
 	bool showMoreInfo;
 	int currentScene;
 	SMIL::MediaClippingTime time;
+	std::vector< GtkWidget* > recentMenuItems;
 
 public:
 
@@ -191,6 +195,7 @@ public:
 	void saveFrame( );
 	void publishPlayList( );
 	void publishFrame( );
+	void loadPlayList( char* );
 
 	// Page related functions
 	void changePageRequest( int page );
@@ -297,6 +302,7 @@ public:
 	{
 		last_directory = value;
 	}
+	void updateRecentFiles();
 
 	// Exit the application
 	bool exitKino( );
