@@ -129,7 +129,7 @@ kino_sound_init( dv_audio_t *audio, kino_sound_t *sound, char const *device )
 		{
 			rate_request = 44100;
 			used = rate_request;
-			printf( ">>> audio at %d failed - forcing resample at %dhz\n", audio->frequency, rate_request );
+			fprintf( stderr, ">>> audio at %d failed - forcing resample at %dhz\n", audio->frequency, rate_request );
 			if ( ioctl( sound->fd, SOUND_PCM_WRITE_RATE, &rate_request ) == -1 )
 			{
 				perror( "SNDCTL_DSP_SPEED 44.1" );
