@@ -1,6 +1,7 @@
 /*
 * page_capture.cc Notebook Firewire Capture Page Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -95,7 +96,7 @@ extern "C"
 */
 
 PageCapture::PageCapture( KinoCommon *common ) : isCapturing( false ), captureMutex( false ),
-		saveAVCActive( false ), avc_enabled( false ), driver_locked( false )
+		avc_enabled( false ), driver_locked( false )
 {
 	std::cerr << "> Creating Capture Page" << std::endl;
 	this->common = common;
@@ -224,7 +225,7 @@ void PageCapture::CheckDevices( )
 
 		// Start the reader thread
 		//std::cerr << "Starting thread" << std::endl;
-		if ( reader->StartThread( avc->getPort() ) )
+		if ( reader->StartThread( ( avc->getPort() < 0 ) ? 0 : avc->getPort() ) )
 		{
 			// The drivers are installed
 			driver_available = true;
@@ -261,7 +262,7 @@ void PageCapture::CheckDevices( )
 			avcStatus = status;
 			avcRetry = 0;
 
-			if ( avc->Timecode( Preferences::getInstance().phyID, t ) )
+			if ( !isCapturing && avc->Timecode( Preferences::getInstance().phyID, t ) )
 			{
 				strncpy( timecode, t, 12 );
 			}
@@ -348,7 +349,7 @@ void PageCapture::applyAVCState( quadlet_t newAVCState )
 			cerr << ">>> AVC enabled " << std::endl;
 
 			// Ensure that we reflect the current avc state when changing gui states
-			avcState = !avcState;
+			avcState = true;
 		}
 
 		if ( avcState != newAVCState )
@@ -392,6 +393,7 @@ void PageCapture::applyAVCState( quadlet_t newAVCState )
 				}
 				else if ( resp3 == AVC1394_VCR_OPERAND_WIND_STOP )
 				{
+					stopCapture();
 					common->toggleComponents( VIDEO_STOP, true );
 				}
 				else if ( resp3 == AVC1394_VCR_OPERAND_WIND_REWIND )
@@ -1118,15 +1120,18 @@ void PageCapture::startCapture()
 				    1, 0UL
 			    };
 
-			saveAVCActive = avc_enabled;
-			if ( saveAVCActive )
+			if ( avc_enabled )
 			{
 				g_nav_ctl.active = FALSE;
 				videoPlay();
 				g_nav_ctl.active = TRUE;
-				gtk_toggle_button_set_active( avcButton, false );
-				applyAVCState( avcStatus );
-				avc_enabled = false;
+				gui_state_was = 2;
+				validComponents = 0;
+				common->activateWidgets( );
+				common->commitComponentState();
+				gtk_widget_set_sensitive( GTK_WIDGET( recordButton ), true );
+				gtk_widget_set_sensitive( GTK_WIDGET( stopButton ), true );
+				gtk_widget_set_sensitive( GTK_WIDGET( muteButton ), true );
 				gtk_widget_set_sensitive( GTK_WIDGET( avcButton ), false );
 				gtk_widget_set_sensitive( GTK_WIDGET( snapshotButton ), false );
 			}
@@ -1194,7 +1199,7 @@ void PageCapture::startCapture()
 					}
 				}
 				common->setStatusBar( _( "Waiting for DV %s%d ..." ),
-				                      ( saveAVCActive ? "" : _( "(press play on camera) " ) ), retry );
+				                      ( avc_enabled ? "" : _( "(press play on camera) " ) ), retry );
 				while ( gtk_events_pending() )
 					gtk_main_iteration();
 				nanosleep( &t, NULL );
@@ -1246,10 +1251,10 @@ void PageCapture::stopCapture()
 		audioOn = FALSE;
 		gtk_toggle_button_set_active( this->muteButton, !audioOn );
 
-		if ( saveAVCActive )
+		if ( avc_enabled )
 		{
-			gtk_toggle_button_set_active( avcButton, saveAVCActive );
-			avc_enabled = saveAVCActive;
+			gtk_toggle_button_set_active( avcButton, avc_enabled );
+			gui_state_was = -1;
 			videoPause( );
 		}
 		common->setStatusBar( "" );
@@ -1516,27 +1521,32 @@ extern "C"
 			// Do with it what needs to be done
 			if ( frame != NULL )
 			{
-				// Play audio
-				if ( audioOn && !critical_mass )
-					if ( writer == NULL || ( writer != NULL && Preferences::getInstance().preview_capture ) )
-						displayer->PutSound( *frame );
-
-				// All access to the writer is protected
-				pthread_mutex_lock( &writerlock );
-				if ( writer != NULL )
+				TimeCode tc;
+				frame->GetTimeCode( tc );
+				if ( frame->IsNormalSpeed() &&
+				     ! ( tc.hour == 0 && tc.min == 0 && tc.sec == 0 && tc.frame == 0 ) )
 				{
-					if ( !writer->WriteFrame( *frame ) )
+					// Play audio
+					if ( audioOn && !critical_mass )
+						if ( writer == NULL || ( writer != NULL && Preferences::getInstance().preview_capture ) )
+							displayer->PutSound( *frame );
+	
+					// All access to the writer is protected
+					pthread_mutex_lock( &writerlock );
+					if ( writer != NULL )
 					{
-						pthread_mutex_unlock( &writerlock );
-						gdk_threads_enter();
-						common->getPageCapture() ->stopCapture();
-						common->setStatusBar( _( "WARNING: Capture stopped. Disk full?" ) );
-						gdk_threads_leave();
+						if ( !writer->WriteFrame( *frame ) )
+						{
+							pthread_mutex_unlock( &writerlock );
+							gdk_threads_enter();
+							common->getPageCapture() ->stopCapture();
+							common->setStatusBar( _( "WARNING: Capture stopped. Disk full?" ) );
+							gdk_threads_leave();
+						}
+						framesCaptured++;
 					}
-					framesCaptured++;
+					pthread_mutex_unlock( &writerlock );
 				}
-				pthread_mutex_unlock( &writerlock );
-
 				// Return the frame for reuse
 				reader->DoneWithFrame( frame );
 
@@ -1614,6 +1624,10 @@ extern "C"
 			{
 				pthread_mutex_unlock( &writerlock );
 			}
+
+			// Release temporarily
+			struct timespec t = { 0, 1000000 }; // 1ms
+			nanosleep( &t, NULL );
 		}
 
 		return NULL;
