@@ -1,6 +1,7 @@
 /*
 * page_capture.h Notebook Firewire Capture Page Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -41,7 +42,6 @@ private:
 	bool	isCapturing;
 	bool	captureMutex;
 	gulong validComponents;
-	bool	saveAVCActive;
 	int gui_state_was;
 
 	/* widgets */
@@ -100,6 +100,10 @@ public:
 
 	void CheckDevices( );
 	void collectFiles( );
+	std::string getHelpPage()
+	{
+		return "capture";
+	}
 
 };
 
