@@ -1,6 +1,7 @@
 /*
 * page_edit.cc Notebook Editor Page Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -53,9 +54,9 @@ extern "C"
 	static void *readThread( void * info );
 	static void *audioThread( void * info );
 	static void *videoThread( void * info );
-	static pthread_t readthread;
-	static pthread_t audiothread;
-	static pthread_t videothread;
+	static pthread_t readthread = 0;
+	static pthread_t audiothread = 0;
+	static pthread_t videothread = 0;
 	static pthread_mutex_t threadlock = PTHREAD_MUTEX_INITIALIZER;
 	static pthread_mutex_t readlock = PTHREAD_MUTEX_INITIALIZER;
 	static int newFrame = 0;
@@ -65,7 +66,6 @@ extern "C"
 
 	static Frame *frameContent[ PLAYBACK_FRAMES ];
 	IEEE1394Writer *writer1394 = NULL;
-	static void TriggerQueueAction( );
 
 	static gboolean doScrub = FALSE;
 	gboolean
@@ -175,9 +175,9 @@ void PageEditor::start()
 		AVC avc;
 		Preferences::getInstance( ).phyID = avc.getNodeId( Preferences::getInstance( ).avcGUID );
 		Preferences::getInstance( ).phyID = avc.isPhyIDValid( Preferences::getInstance( ).phyID );
-		if ( Preferences::getInstance( ).phyID >= 0 && !writer1394 )
+		if ( !writer1394 )
 			writer1394 = new iec61883Writer(
-			             avc.getPort(),
+			             ( avc.getPort() < 0 ) ? 0 : avc.getPort(),
 			             Preferences::getInstance().channel,
 			             2 );
 #else
@@ -230,7 +230,8 @@ void PageEditor::clean()
 {
 	cerr << ">> Leaving Editor" << endl;
 	stopNavigator();
-	delete this->displayer;
+	delete displayer;
+	displayer = 0;
 	delete writer1394;
 	writer1394 = NULL;
 	for ( int i = 0; i < PLAYBACK_FRAMES; i ++ )
@@ -261,7 +262,6 @@ void PageEditor::stopNavigator()
 	{
 		g_nav_ctl.active = FALSE;
 		gdk_threads_leave();
-		TriggerQueueAction( );
 		pthread_join( readthread, NULL );
 		pthread_join( audiothread, NULL );
 		pthread_join( videothread, NULL );
@@ -284,7 +284,9 @@ void PageEditor::movedToFrame( int frame )
 	}
 	else
 	{
-		lastFrame = newFrame = frame;
+		stopNavigator();
+		common->g_currentFrame = frame;
+		startNavigator();
 	}
 }
 
@@ -308,7 +310,7 @@ void PageEditor::showFrame( int i, gboolean no_audio )
 		common->getPlayList() ->GetFrame( i, *frame );
 		common->showFrameInfo( i );
 		if ( writer1394 != NULL )
-			writer1394->SendFrame( *frame ) ;
+			writer1394->SendFrame( *frame, false ) ;
 		getFrameDisplayer() ->Put( *frame, GTK_WIDGET( frameArea ), no_audio );
 		lastFrameShown = i;
 		GetFramePool( ) ->DoneWithFrame( frame );
@@ -356,23 +358,22 @@ void PageEditor::videoStartOfScene()
 void PageEditor::videoRewind()
 {
 	common->toggleComponents( common->getComponentState(), false );
-
+	stopNavigator();
 	// Toggle Rewind state
-	if ( g_nav_ctl.step != -10 )
+	if ( g_nav_ctl.step == -10 )
+	{
+		g_nav_ctl.step = 1;
+		g_nav_ctl.rate = 1;
+		common->toggleComponents( VIDEO_REWIND, false );
+		common->toggleComponents( VIDEO_STOP, true );
+	}
+	else
 	{
 		common->toggleComponents( VIDEO_REWIND, true );
 		g_nav_ctl.step = -10;
 		g_nav_ctl.rate = 0;
 		startNavigator();
 	}
-	else
-	{
-		stopNavigator();
-		g_nav_ctl.step = 1;
-		g_nav_ctl.rate = 1;
-		common->toggleComponents( VIDEO_REWIND, false );
-		common->toggleComponents( VIDEO_STOP, true );
-	}
 }
 
 /** Move one frame back. If the navigator is active, then this action toggles
@@ -384,15 +385,16 @@ void PageEditor::videoBack(int step)
 	common->toggleComponents( common->getComponentState(), false );
 	if ( g_nav_ctl.active )
 	{
+		stopNavigator();
 		if ( g_nav_ctl.step != -1 )
 		{
 			g_nav_ctl.step = -1;
 			g_nav_ctl.rate = 1;
 			common->toggleComponents( VIDEO_BACK, true );
+			startNavigator();
 		}
 		else
 		{
-			stopNavigator();
 			g_nav_ctl.step = 1;
 			g_nav_ctl.rate = 0;
 			common->toggleComponents( VIDEO_BACK, false );
@@ -416,6 +418,7 @@ void PageEditor::videoPlay()
 	if ( g_nav_ctl.active == FALSE || g_nav_ctl.step != 1 )
 	{
 		common->toggleComponents( VIDEO_PLAY, true );
+		stopNavigator();
 		g_nav_ctl.step = 1;
 		g_nav_ctl.rate = 1;
 		startNavigator();
@@ -433,9 +436,7 @@ void PageEditor::videoPlay()
 
 void PageEditor::videoPause()
 {
-	common->toggleComponents( common->getComponentState(), false );
-	common->toggleComponents( VIDEO_STOP, true );
-	stopNavigator();
+	videoStop();
 }
 
 /** Stop.
@@ -446,6 +447,8 @@ void PageEditor::videoStop()
 	common->toggleComponents( common->getComponentState(), false );
 	common->toggleComponents( VIDEO_STOP, true );
 	stopNavigator();
+	g_nav_ctl.step = 0;
+	g_nav_ctl.rate = 0;
 }
 
 /** Move one frame forward. If the navigator is active, then this action puts
@@ -458,9 +461,10 @@ void PageEditor::videoForward(int step)
 	common->toggleComponents( common->getComponentState(), false );
 	if ( g_nav_ctl.active && g_nav_ctl.step != 1 )
 	{
+		common->toggleComponents( VIDEO_FORWARD, true );
+		stopNavigator();
 		g_nav_ctl.step = 1;
 		g_nav_ctl.rate = 1;
-		common->toggleComponents( VIDEO_FORWARD, true );
 		startNavigator();
 	}
 	else
@@ -478,7 +482,7 @@ void PageEditor::videoForward(int step)
 void PageEditor::videoFastForward()
 {
 	common->toggleComponents( common->getComponentState(), false );
-
+	stopNavigator();
 	if ( g_nav_ctl.step != 10 )
 	{
 		common->toggleComponents( VIDEO_FAST_FORWARD, true );
@@ -488,7 +492,6 @@ void PageEditor::videoFastForward()
 	}
 	else
 	{
-		stopNavigator();
 		g_nav_ctl.step = 1;
 		g_nav_ctl.rate = 1;
 		common->toggleComponents( VIDEO_FAST_FORWARD, false );
@@ -505,41 +508,39 @@ void PageEditor::videoFastForward()
 
 void PageEditor::videoShuttle( int angle )
 {
-	int speedTable[] = { 
+	int frames_sec = _getOneSecond();
+	int speedTable[] = {
 			0,
 			8,  10, 15, 20, 33, 50, 75,
 			100,
 			200, 300, 400, 500, 800, 1200,
-			( _getOneSecond() * 100 ) };
+			( frames_sec * 100 ) };
 	char s[ 64 ];
 
-//	stopNavigator( );
-
 	if ( angle < -15 )
 		angle = -15;
 	if ( angle > 15 )
 		angle = 15;
 
 	int speed = speedTable[ ( angle < 0 ) ? -angle : angle ] * ( ( angle < 0 ) ? -1 : 1 );
-	if ( speed == 0 )
+	if ( speed == 0 || g_nav_ctl.step != speed / 100 || g_nav_ctl.rate != 100 / speed )
 	{
-stopNavigator( );
-		common->keyboardFeedback( "", "" );
-		common->toggleComponents( VIDEO_STOP, true );
-		return ;
+		stopNavigator( );
+		if ( speed == 0 )
+		{
+			common->keyboardFeedback( "", "" );
+			common->toggleComponents( VIDEO_STOP, true );
+		}
+		else
+		{
+			g_nav_ctl.step = speed / 100;
+			g_nav_ctl.rate = 100 / speed;
+			g_nav_ctl.subframe = 0;
+			startNavigator();
+			snprintf( s, 63, _( "Shuttle %+.1f fps" ), ( float ) speed / 100.0 * frames_sec );
+			common->keyboardFeedback( "", s );
+		}
 	}
-
-pthread_mutex_lock( &readlock );
-	g_nav_ctl.step = speed / 100;
-	g_nav_ctl.rate = 100 / speed;
-	g_nav_ctl.subframe = 0;
-pthread_mutex_unlock( &readlock );
-
-	snprintf( s, 63, _( "Shuttle %+.1f fps" ), ( float ) speed / 100.0 * ( float ) _getOneSecond() );
-	common->keyboardFeedback( "", s );
-
-if ( g_nav_ctl.active == FALSE )
-	startNavigator();
 }
 
 /** Move to the start of the next scene.
@@ -891,7 +892,7 @@ gboolean PageEditor::processCommand( char *command )
 	{
 		start = common->getPlayList() ->FindStartOfScene( common->g_currentFrame );
 		end = start;
-		for ( ; count >= 1 && end < common->getPlayList() ->GetNumFrames() - 1; count -- )
+		for ( ; count >= 1 && end <= common->getPlayList() ->GetNumFrames() - 1; count -- )
 		{
 			end = common->getPlayList() ->FindEndOfScene( end );
 			end ++;
@@ -1273,6 +1274,7 @@ void PageEditor::DeleteFrames( int first, int last )
 
 void PageEditor::showFrame( int position, Frame& frame )
 {
+	gdk_threads_enter();
 	if ( common->getPlayList() ->GetNumFrames() == 0 )
 	{
 		common->loadSplash( frameArea );
@@ -1281,8 +1283,16 @@ void PageEditor::showFrame( int position, Frame& frame )
 	{
 		DrawBar( position );
 		getFrameDisplayer() ->Put( frame, GTK_WIDGET( frameArea ), TRUE );
-		showFrameInfo( common->g_currentFrame, frame );
+		showFrameInfo( position, frame );
+		common->g_currentFrame = position;
 	}
+	gdk_flush();
+	if ( position >= ( common->getPlayList()->GetNumFrames() - 1 ) ||
+		( position <= 0 && ( g_nav_ctl.step < 0 || g_nav_ctl.rate < 0 ) ) )
+	{
+		common->videoStop();
+	}
+	gdk_threads_leave();
 }
 
 
@@ -1396,7 +1406,6 @@ extern "C"
 	// frameNumber and frameContent.
 
 	static int frameNumber[ PLAYBACK_FRAMES ];
-
 	static int pending = 0;
 	static int head = -1;
 	static int tail = -1;
@@ -1412,74 +1421,6 @@ extern "C"
 		tail = 0;
 	}
 
-	static pthread_mutex_t condition_queue_mutex = PTHREAD_MUTEX_INITIALIZER;
-	static pthread_cond_t condition_queue = PTHREAD_COND_INITIALIZER;
-	static pthread_mutex_t condition_audio_mutex = PTHREAD_MUTEX_INITIALIZER;
-	static pthread_cond_t condition_audio = PTHREAD_COND_INITIALIZER;
-	static pthread_mutex_t condition_video_mutex = PTHREAD_MUTEX_INITIALIZER;
-	static pthread_cond_t condition_video = PTHREAD_COND_INITIALIZER;
-
-	static void WaitForQueueAction( )
-	{
-		pthread_mutex_lock( &condition_queue_mutex );
-		pthread_cond_wait( &condition_queue, &condition_queue_mutex );
-		pthread_mutex_unlock( &condition_queue_mutex );
-	}
-
-	static void TriggerQueueAction( )
-	{
-		pthread_mutex_lock( &condition_queue_mutex );
-		pthread_cond_signal( &condition_queue );
-		pthread_mutex_unlock( &condition_queue_mutex );
-	}
-
-#if 0 //not being used at the moment
-	static void WaitForAudioAction( int lastPosition )
-	{
-		if ( tail == -1 || frameNumber[ tail ] == lastPosition )
-		{
-			pthread_mutex_lock( &condition_audio_mutex );
-			pthread_cond_wait( &condition_audio, &condition_audio_mutex );
-			pthread_mutex_unlock( &condition_audio_mutex );
-		}
-	}
-#endif
-
-	static void TriggerAudioAction( )
-	{
-		pthread_mutex_lock( &condition_audio_mutex );
-		pthread_cond_signal( &condition_audio );
-		pthread_mutex_unlock( &condition_audio_mutex );
-	}
-
-	static void WaitForVideoAction( int lastPosition )
-	{
-		if ( head == -1 || frameNumber[ head ] == lastPosition )
-		{
-			pthread_mutex_lock( &condition_video_mutex );
-			pthread_cond_wait( &condition_video, &condition_video_mutex );
-			pthread_mutex_unlock( &condition_video_mutex );
-		}
-	}
-
-	static void TriggerVideoAction( )
-	{
-		pthread_mutex_lock( &condition_video_mutex );
-		pthread_cond_signal( &condition_video );
-		pthread_mutex_unlock( &condition_video_mutex );
-	}
-	
-	static void on_end_of_movie( )
-	{
-		if ( common->g_currentFrame >= common->getPlayList()->GetNumFrames() - 1 ||
-			( common->g_currentFrame <= 0 && ( g_nav_ctl.step < 0 || g_nav_ctl.rate < 0 ) ) )
-		{
-			gdk_threads_enter();
-			common->videoStop();
-			gdk_threads_leave();
-		}
-	}
-
 	/** This function carries out the read ahead and is responsible for obtaining
 		the each frame. audioThread monitors the queue (by checking mainly on pending)
 		and plays the audio of each frame. videoThread picks up the last tail and
@@ -1509,7 +1450,6 @@ extern "C"
 		// Loop while active
 		while ( ctl->active )
 		{
-
 			// Calculate time for next frame
 			start.tv_usec += time_per_frame;
 			if ( start.tv_usec >= 1000000 )
@@ -1555,46 +1495,33 @@ extern "C"
 			if ( pending >= PLAYBACK_FRAMES - 10 )
 			{
 				// Queue is full.. do nothing
-				WaitForQueueAction( );
-				//struct timespec t = { 0, 0 };
-				//nanosleep( &t, NULL );
 				playing = true;
+				struct timespec t = { 0, 1000000 }; // 1ms
+				nanosleep( &t, NULL );
 			}
 			else
 			{
-				pthread_mutex_lock( &threadlock );
-				int new_head = ( head + 1 ) % PLAYBACK_FRAMES;
-				// cerr << "captured pending = " << pending << " new_head = " << new_head << " tail = " << tail << endl;
-				pthread_mutex_unlock( &threadlock );
-
-				frameNumber[ new_head ] = newFrame;
-				common->getPlayList() ->GetFrame( frameNumber[ new_head ], *frameContent[ new_head ] );
-				pthread_mutex_lock( &threadlock );
-				pending ++;
-				head = new_head;
-				pthread_mutex_unlock( &threadlock );
-				
-				TriggerAudioAction( );
-
+				frameNumber[ head ] = newFrame;
+				common->getPlayList() ->GetFrame( frameNumber[ head ], *frameContent[ head ] );
 				if ( writer1394 != NULL )
-					writer1394->SendFrame( *frameContent[ tail ] );
-					
+					writer1394->SendFrame( *frameContent[ head ], false );
+				
+				pthread_mutex_lock( &threadlock );
+				++pending;
+				head = ( head + 1 ) % PLAYBACK_FRAMES;
+				pthread_mutex_unlock( &threadlock );
+
 				// We have a frame, so start playing
 				playing = true;
 
 				// Render all frames if requested
 				if ( ( ! prefs.dropFrame || ! prefs.enableAudio ) && ctl->active )
 				{
-					tail = ( tail + 1 ) % PLAYBACK_FRAMES;
-					common->g_currentFrame = frameNumber[ tail ];
-					pending --;
-
-					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 ) )
+					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 || ctl->rate < -1 ) )
 						common->getPageEditor() ->getFrameDisplayer() ->PutSound( *frameContent[ tail ] );
-					gdk_threads_enter();
 					common->getPageEditor() ->showFrame( frameNumber[ tail ], *frameContent[ tail ] );
-					gdk_flush();
-					gdk_threads_leave();
+					tail = ( tail + 1 ) % PLAYBACK_FRAMES;
+					--pending;
 
 					// Determine how far from the next frame we really are
 					gettimeofday( &end, NULL );
@@ -1616,20 +1543,16 @@ extern "C"
 					{
 						gettimeofday( &start, NULL );
 					}
-				} 
+				}
 
 				// update the lastFrame and lastPos variables
 				lastFrame = newFrame;
 
 				// Incrment the frame count
 				countFrames ++;
-				
-				on_end_of_movie();
 			}
 		}
-
-		// cerr << ">>> Ending reading thread" << endl;
-		TriggerAudioAction( );
+		// cerr << ">>> Ending read thread " << countFrames << endl;
 
 		return NULL;
 	}
@@ -1646,31 +1569,21 @@ extern "C"
 
 		while ( ctl->active )
 		{
-
 			// Only do this when drop frames is active (otherwise the audio thread takes care of it)
 			if ( prefs.dropFrame && prefs.enableAudio )
 			{
-
-//				if ( ctl->step > 1 || ctl->step < -1 )
-//					WaitForAudioAction( lastFrame );
-
-				// Obtain the position of the last released frame
-				pthread_mutex_lock( &threadlock );
-				int new_tail = ( tail + 1 ) % PLAYBACK_FRAMES;
-				// cerr << "playing  pending = " << pending << " head = " << head << " new_tail = " << new_tail << endl;
-				pthread_mutex_unlock( &threadlock );
-
 				// As long as its different to the previous one, then display it
 				if ( playing && pending > 0 )
 				{
-
-					// Move the main frame position to our newly derived location
-					common->g_currentFrame = frameNumber[ tail ];
-
 					// Play the audio
-					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 ) )
+					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 || ctl->rate < -1 ) )
 					{
 						common->getPageEditor() ->getFrameDisplayer() ->PutSound( *frameContent[ tail ] );
+						if ( pending < 10 )
+						{
+							struct timespec t = { 0, 1000000 }; // 1ms
+							nanosleep( &t, NULL );
+						}
 					}
 					else if ( ctl->step == 0 && !prefs.audioScrub )
 					{
@@ -1686,21 +1599,18 @@ extern "C"
 
 					// Move the tail
 					pthread_mutex_lock( &threadlock );
-					pending --;
-					tail = new_tail;
+					--pending;
+					tail = ( tail + 1 ) % PLAYBACK_FRAMES;;
 					pthread_mutex_unlock( &threadlock );
 
-					if ( pending < PLAYBACK_FRAMES / 2 )
-						TriggerQueueAction( );
-
 					// Start showing frames from this point onwards
 					showing = true;
-
-					TriggerVideoAction( );
 				}
 				else
 				{
 					playing = false;
+					struct timespec t = { 0, 1000000 }; // 1ms
+					nanosleep( &t, NULL );
 				}
 			}
 			else
@@ -1710,9 +1620,7 @@ extern "C"
 				return NULL;
 			}
 		}
-
-		TriggerVideoAction( );
-		// cerr << ">>> Ending video thread" << endl;
+		// cerr << ">>> Ending audio thread" << endl;
 
 		return NULL;
 	}
@@ -1731,19 +1639,13 @@ extern "C"
 #endif
 
 		// cerr << ">>> Starting video thread" << endl;
-
 		while ( ctl->active )
 		{
-
 			// Only do this when drop frames is active (otherwise the audio thread takes care of it)
 			if ( prefs.dropFrame && prefs.enableAudio )
 			{
-
-				WaitForVideoAction( lastFrame );
-
-				if ( showing && pending > 0 )
+				if ( showing && pending > 0 && ctl->active )
 				{
-
 					// Store the current tail
 					int my_tail = tail;
 
@@ -1764,17 +1666,12 @@ extern "C"
 							lastFrame = frameNumber[ my_tail ];
 						}
 #endif
-
 						// Show this frame
-						gdk_threads_enter();
-						if ( ctl->active )
-						{
-							common->getPageEditor() ->showFrame( frameNumber[ my_tail ], *frameContent[ my_tail ] );
-							gdk_flush();
-						}
-						gdk_threads_leave();
+						common->getPageEditor() ->showFrame( frameNumber[ my_tail ], *frameContent[ my_tail ] );
 					}
 				}
+				struct timespec t = { 0, 1000000 }; // 1ms
+				nanosleep( &t, NULL );
 			}
 			else
 			{
@@ -1787,7 +1684,6 @@ extern "C"
 #ifdef PLAY_WITH_STATS
 		cerr << ">>>> Video stopped: Dropped " << dropped << " in " << count << " frames - " << ( double ) ( ( ( double ) dropped / ( double ) count ) * 100.0 ) << "%" << endl;
 #endif
-
 		// cerr << ">>> Ending video thread" << endl;
 
 		return NULL;
