@@ -1,6 +1,7 @@
 /*
 * page_edit.h Notebook Editor Page Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -107,7 +108,7 @@ public:
 	void DeleteFrames( int, int );
 	std::string getHelpPage()
 	{
-		return "edit.html";
+		return "edit";
 	}
 };
 
