@@ -1,6 +1,6 @@
 /*
 * page_export_stills.cc Notebook Firewire/AVI/Still Frame Export Page Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -97,8 +97,7 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 	AudioInfo info;
 	frame.GetAudioInfo( info );
 	short channels = info.channels;
-	int inputFrequency = info.frequency;
-	int outputFrequency = inputFrequency;
+	int outputFrequency = info.frequency;
 	bool splitScene = gtk_toggle_button_get_active( splitCheck );
 
 	// Determine requested output frequency
@@ -117,40 +116,34 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 	menuValue = g_list_index ( GTK_MENU_SHELL ( audioFormat ) ->children, active_item );
 
 	gchar *outputtemplate = NULL;
-	int resampleRate = 0;
 	kino_audio_pipe type = PIPE_AUDIO_WAV;
 
 	if ( menuValue == 0 )
 	{
 		outputtemplate = g_strdup_printf( "%s%%s.wav", file );
-		resampleRate = outputFrequency;
 		type = PIPE_AUDIO_WAV;
 	}
 	else if ( menuValue == 1 )
 	{
-		outputtemplate = g_strdup_printf( "| sox -t .raw -r %d -c 2 -w -s - -r %d \'%s\'%%s.wav",
-		         inputFrequency, outputFrequency, file );
-		resampleRate = inputFrequency;
+		outputtemplate = g_strdup_printf( "| sox -t .raw -r %d -c 2 -w -s - \'%s\'%%s.wav",
+		         outputFrequency, file );
 		type = PIPE_AUDIO_WAV;
 	}
 	else if ( menuValue == 2 )
 	{
 		outputtemplate = g_strdup_printf( "| mp2enc -r %d -o \'%s\'%%s.mp2",
 		         outputFrequency, file );
-		resampleRate = inputFrequency;
 		type = PIPE_AUDIO_WAV;
 	}
 	else if ( menuValue == 3 )
 	{
 		outputtemplate = g_strdup_printf( "| lame -r -s %d -x - -o \'%s\'%%s.mp3",
-		         inputFrequency / 1000, file );
-		resampleRate = inputFrequency;
+		         outputFrequency / 1000, file );
 		type = PIPE_AUDIO_PCM;
 	}
 	else if ( menuValue == 4 )
 	{
 		outputtemplate = g_strdup_printf( "| oggenc - -o \'%s\'%%s.ogg", file );
-		resampleRate = inputFrequency;
 		type = PIPE_AUDIO_WAV;
 	}
 
@@ -181,7 +174,7 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 		full = g_strdup_printf( outputtemplate, sceneString );
 
 		// output this scene
-		bool active = outputPipe->OpenAudio( full, channels, resampleRate, 2 );
+		bool active = outputPipe->OpenAudio( full, channels, outputFrequency, 2 );
 		g_free( full );
 		
 		if ( active )
@@ -192,13 +185,13 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 			innerLoopUpdate( sceneBegin, begin, end, every );
 
 			// Determine correct amount of audio for duration
-			double adjustedRate = calculateAdjustedRate( playlist, resampleRate, sceneBegin, sceneEnd, every );
+			double adjustedRate = calculateAdjustedRate( playlist, outputFrequency, sceneBegin, sceneEnd, every );
 			if ( ! adjustedRate )
 				status = EXPORT_RESULT_ABORT;
 
 			// Setup a resampler
 			AsyncAudioResample<int16_ne_t,int16_le_t>* resampler = new AsyncAudioResample<int16_ne_t,int16_le_t>(
-				AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, playlist, resampleRate, sceneBegin, sceneEnd, every );
+				AUDIO_RESAMPLE_SRC_SINC_BEST_QUALITY, playlist, outputFrequency, sceneBegin, sceneEnd, every );
 			if ( resampler->IsError() )
 			{
 				std::cerr << ">>> Resampler error: " << resampler->GetError() << std::endl;
@@ -213,7 +206,7 @@ ExportAudio::doExport( PlayList * playlist, int begin, int end, int every,
 			{
 				innerLoopUpdate( i, begin, end, every );
 
-				int requestedSamples = frame.CalculateNumberSamples( resampleRate, j );
+				int requestedSamples = frame.CalculateNumberSamples( outputFrequency, j );
 				int nsamples = resampler->Process( adjustedRate, requestedSamples );
 				if ( nsamples > 0 && !outputPipe->OutputAudioFrame( (int16_t *)resampler->GetOutput(), nsamples * channels * sizeof(int16_t) ) )
 				{
