@@ -1,6 +1,7 @@
 /*
-* page_magick.h -- Image Magick Integration
+* page_magick.h -- FX Page
 * Copyright (C) 2002 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2002-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -28,6 +29,7 @@
 #include "kino_common.h"
 #include "page.h"
 #include "oss.h"
+#include "kino_extra.h"
 
 /** Simple class wrapping for the shared object functions.
 */
@@ -109,7 +111,7 @@ public:
 /** Magick page class
 */
 
-class PageMagick : public Page
+class PageMagick : public Page, public KeyFrameController
 {
 public:
 	PageMagick( KinoCommon *common );
@@ -128,6 +130,12 @@ public:
 	void videoEndOfMovie();
 	void videoPause();
 	void videoStop();
+	void videoPreviousScene();
+	void videoNextScene();
+	void videoStartOfScene()
+		{ videoPreviousScene(); }
+	void videoEndOfScene()
+		{ videoNextScene(); }
 	void showFrameInfo( int frame_number );
 	void showFrameInfo( int frame_number, int duration );
 	void movedToFrame( int frame_number );
@@ -137,12 +145,14 @@ public:
 	gboolean processCommand( char *cmd );
 	std::string getHelpPage()
 	{
-		return "fx.html";
+		return "FX";
 	}
 
 	// PageMagick methods
 	void PreviewFrame();
 	void StartPreview();
+	void AudioThread();
+	void VideoThread();
 	void StartRender();
 	void Stop();
 	void StopPreview();
@@ -164,6 +174,21 @@ public:
 	{
 		return repainting;
 	}
+	bool IsPreviewing() const
+	{
+		return previewing;
+	}
+	void SetKeyFrameControllerClient( KeyFrameControllerClient* client )
+	{
+		keyFrameControllerClient = client;
+	}
+
+	// KeyFrameController methods
+	void ShowCurrentStatus( double position, frame_type type, bool hasPrev, bool hasNext );
+	double GetCurrentPosition( );
+
+	// KeyFrameController callbacks
+	void OnKeyFrameControllerKeyChanged( GtkToggleButton* togglebutton );
 
 	// Used by signal handlers
 	void OnTimeRangeChanged();
@@ -199,13 +224,18 @@ private:
 	void PlayAudio( int16_t *buffers[], int, int, int, bool );
 
 	// Video structures
-	void ShowImage( GtkWidget *area, uint8_t *pixels, int, int );
+	void ShowImage( GtkWidget *area, uint8_t *pixels, int, int, bool );
 
 	PluginCollection plugins;
 	string last_fx_file;
 	double startTime;
 	double pauseTime;
 	double nextUpdateTime;
+	KeyFrameControllerClient* keyFrameControllerClient;
+	bool isGuiLocked;
+	bool isPreviousScene;
+	bool isNextScene;
+	double newPosition;
 };
 
 #endif
