@@ -1,6 +1,7 @@
 /*
 * page_timeline.cc Notebook Timeline Page Object
 * Copyright (C) 2001 Charles Yates <charles.yates@pandora.be>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -192,7 +193,7 @@ PageTimeline::PageTimeline( KinoCommon *common ) :
 
 void PageTimeline::start()
 {
-	cout << ">> Starting timeline" << endl;
+	std::cerr << ">> Starting timeline" << std::endl;
 
 	if ( common->getPlayList() ->GetNumFrames() > 0 )
 	{
@@ -234,7 +235,7 @@ void PageTimeline::start()
 
 void PageTimeline::selectScene( int i )
 {
-	cout << "Time line scene " << i << endl;
+	std::cerr << "Time line scene " << i << std::endl;
 	GtkWidget *start_spin = lookup_widget( common->getWidget(), "start_spin" );
 	GtkWidget *end_spin = lookup_widget( common->getWidget(), "end_spin" );
 	GtkAdjustment *adjust = gtk_spin_button_get_adjustment( GTK_SPIN_BUTTON( start_spin ) );
@@ -386,7 +387,7 @@ void* PageTimeline::Thread()
 							frame.GetWidth(), frame.GetHeight(), frame.GetWidth() * 3, NULL, NULL );
 	
 			// Rescale
-			GdkPixbuf *scaled = gdk_pixbuf_scale_simple( image, 90, 68, GDK_INTERP_NEAREST );
+			GdkPixbuf *scaled = gdk_pixbuf_scale_simple( image, 90, frame.IsWide() ? 51 : 68, GDK_INTERP_NEAREST );
 	
 			// Set the label
 			char label[ 256 ] = "";
