@@ -1,7 +1,6 @@
 /*
-* page_edit.h Notebook Trim Page Object
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
-* Custom GTK+ Trim Control by Ronald Bultje
+* page_trim.cc Notebook Trim Page Object
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -116,8 +115,7 @@ extern "C"
 			{
 				if ( value > common->getPageTrim() ->getOutPoint() )
 					gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_OUT ], value );
-				else
-					common->getPageTrim() ->setInPoint( value );
+				common->getPageTrim() ->setInPoint( value );
 			}
 			skipPosUpdate = TRUE;
 			common->getPageTrim() ->movedToFrame( value );
@@ -134,8 +132,7 @@ extern "C"
 				{
 					if ( value <= common->getPageTrim() ->getInPoint() )
 						gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_IN ], value - 1 );
-					else
-						common->getPageTrim() ->setOutPoint( value );
+					common->getPageTrim() ->setOutPoint( value );
 
 					skipPosUpdate = TRUE;
 					gtk_spin_button_set_value( spin_in, ( gfloat ) diff );
@@ -152,8 +149,7 @@ extern "C"
 			{
 				if ( value < common->getPageTrim() ->getInPoint() )
 					gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_IN ], value );
-				else
-					common->getPageTrim() ->setOutPoint( value );
+				common->getPageTrim() ->setOutPoint( value );
 			}
 			skipPosUpdate = TRUE;
 			common->getPageTrim() ->movedToFrame( value );
@@ -555,9 +551,9 @@ void PageTrim::start()
 		AVC avc;
 		Preferences::getInstance( ).phyID = avc.getNodeId( Preferences::getInstance( ).avcGUID );
 		Preferences::getInstance( ).phyID = avc.isPhyIDValid( Preferences::getInstance( ).phyID );
-		if ( Preferences::getInstance( ).phyID >= 0 && !writer1394 )
+		if ( !writer1394 )
 			writer1394 = new iec61883Writer(
-			             avc.getPort(),
+			             ( avc.getPort() < 0 ) ? 0 : avc.getPort(),
 			             Preferences::getInstance().channel,
 			             2 );
 #endif
@@ -632,7 +628,8 @@ void PageTrim::clean()
 {
 	cerr << ">> Leaving Trimmer" << endl;
 	stopNavigator();
-	delete this->displayer;
+	delete displayer;
+	displayer = 0;
 	delete writer1394;
 	writer1394 = NULL;
 	saveScene();
@@ -990,6 +987,7 @@ void PageTrim::movedToFrame( int frame )
 		else
 		{
 			lastFrame = newFrame = pos;
+			startNavigator();
 		}
 	}
 }
@@ -1016,8 +1014,9 @@ void PageTrim::showFrame( int i, gboolean no_audio )
 		playlist.GetFrame( i, frame );
 		common->showFrameInfo( i );
 		if ( writer1394 != NULL )
-			writer1394->SendFrame( frame );
-		getFrameDisplayer() ->Put( frame, GTK_WIDGET( frameArea ), no_audio );
+			writer1394->SendFrame( frame, false );
+		if ( GTK_WIDGET_REALIZED( frameArea ) && getFrameDisplayer() )
+			getFrameDisplayer() ->Put( frame, GTK_WIDGET( frameArea ), no_audio );
 		lastPos = i;
 		gtk_adjustment_set_value( trim_adj[ TRIM_ADJ_POS ], ( gfloat ) i );
 		GetFramePool( ) ->DoneWithFrame( &frame );
@@ -1217,46 +1216,44 @@ void PageTrim::videoFastForward()
 
 	Bi-directionaly variable-speed playback.
 
-	\param angle A number from -7 (fastest reverse) to 7 (fastest forward), 0=stop
+	\param angle A number from -15 (fastest reverse) to 15 (fastest forward), 0=stop
 */
 
 void PageTrim::videoShuttle( int angle )
 {
+	int frames_sec = _getOneSecond();
 	int speedTable[] = {
 			0,
 			8,  10, 15, 20, 33, 50, 75,
 			100,
 			200, 300, 400, 500, 800, 1200,
-			( _getOneSecond() * 100 ) };
+			( frames_sec * 100 ) };
 	char s[ 64 ];
 
-//	stopNavigator( );
-
 	if ( angle < -15 )
 		angle = -15;
 	if ( angle > 15 )
 		angle = 15;
 
 	int speed = speedTable[ ( angle < 0 ) ? -angle : angle ] * ( ( angle < 0 ) ? -1 : 1 );
-	if ( speed == 0 )
+	if ( speed == 0 || g_nav_ctl.step != speed / 100 || g_nav_ctl.rate != 100 / speed )
 	{
-stopNavigator( );
-		common->keyboardFeedback( "", "" );
-common->toggleComponents( VIDEO_STOP, true );
-		return ;
+		stopNavigator( );
+		if ( speed == 0 )
+		{
+			common->keyboardFeedback( "", "" );
+			common->toggleComponents( VIDEO_STOP, true );
+		}
+		else
+		{
+			g_nav_ctl.step = speed / 100;
+			g_nav_ctl.rate = 100 / speed;
+			g_nav_ctl.subframe = 0;
+			startNavigator();
+			snprintf( s, 63, _( "Shuttle %+.1f fps" ), ( float ) speed / 100.0 * frames_sec );
+			common->keyboardFeedback( "", s );
+		}
 	}
-
-pthread_mutex_lock( &readlock );
-	g_nav_ctl.step = speed / 100;
-	g_nav_ctl.rate = 100 / speed;
-	g_nav_ctl.subframe = 0;
-pthread_mutex_unlock( &readlock );
-
-	snprintf( s, 63, "Shuttle %+.1f fps", ( float ) speed / 100.0 * ( float ) _getOneSecond() );
-	common->keyboardFeedback( "", s );
-
-if ( g_nav_ctl.active == FALSE )
-	startNavigator();
 }
 
 /** Move to the end of the current scene.
@@ -1672,14 +1669,16 @@ gboolean PageTrim::processCommand( char *command )
 	else if ( strcmp( real, "dd" ) == 0 )
 	{
 		end_orig = start_orig;
-		for ( ; count >= 1 && end_orig < common->getPlayList() ->GetNumFrames() - 1; count -- )
+		for ( ; count >= 1 && end_orig <= common->getPlayList() ->GetNumFrames() - 1; count -- )
 		{
 			end_orig = common->getPlayList() ->FindEndOfScene( end_orig );
 			end_orig ++;
 		}
 		common->getPageEditor()->CopyFrames( start_orig, end_orig - 1 );
 		common->getPageEditor()->DeleteFrames( start_orig, end_orig - 1 );
-		common->moveToFrame( start_orig - 1 );
+		if ( start_orig > common->getPlayList() ->GetNumFrames() - 1 )
+			start_orig = common->getPlayList() ->GetNumFrames() - 1;
+		common->moveToFrame( start_orig );
 		common->keyboardFeedback( cmd, _( "Cut current scene" ) );
 		common->getPageEditor()->DrawBar( common->g_currentFrame );
 		cmd[ 0 ] = 0;
@@ -2079,10 +2078,7 @@ extern "C"
 			{
 				// Queue is full.. do nothing
 				playing = true;
-				struct timespec t =
-				    {
-					    0, 0
-				    };
+				struct timespec t = { 0, 1000000 }; // 1ms
 				nanosleep( &t, NULL );
 			}
 			else if ( pending < PLAYBACK_FRAMES )
@@ -2101,7 +2097,7 @@ extern "C"
 				pthread_mutex_unlock( &threadlock );
 
 				if ( writer1394 != NULL )
-					writer1394->SendFrame( *frameContent[ tail ] );
+					writer1394->SendFrame( *frameContent[ new_head ], false );
 
 				// Inform the audio thread that it's got something to work with
 				playing = true;
@@ -2118,7 +2114,7 @@ extern "C"
 					common->getPageTrim() ->setPosition( frameNumber[ tail ] );
 
 					// If audio is enabled and some other conditions
-					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 ) )
+					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 || ctl->rate < -1 ) )
 						common->getPageTrim() ->getFrameDisplayer() ->PutSound( *frameContent[ tail ] );
 
 					// Show the corresponding image
@@ -2193,16 +2189,13 @@ extern "C"
 					common->getPageTrim() ->setPosition( frameNumber[ tail ] );
 
 					// Play the audio
-					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 ) )
+					if ( prefs.enableAudio && ( frameNumber[ tail ] != lastFrame || ctl->rate > 1 || ctl->rate < -1 ) )
 					{
 						common->getPageTrim() ->getFrameDisplayer() ->PutSound( *frameContent[ tail ] );
 						if ( pending < 10 )
 						{
 							// Give the reader more time
-							struct timespec t =
-							    {
-								    0, 0
-							    };
+							struct timespec t = { 0, 1000000 }; // 1ms
 							nanosleep( &t, NULL );
 						}
 					}
@@ -2230,10 +2223,7 @@ extern "C"
 				}
 				else
 				{
-					struct timespec t =
-					    {
-						    0, 0
-					    };
+					struct timespec t = { 0, 1000000 }; // 1ms
 					nanosleep( &t, NULL );
 				}
 			}
@@ -2305,14 +2295,10 @@ extern "C"
 							gdk_threads_leave();
 						}
 					}
-
-					// Always release after showing a frame (hmm...)
-					struct timespec t =
-					    {
-						    0, 0
-					    };
-					nanosleep( &t, NULL );
 				}
+				// Release thread
+				struct timespec t = { 0, 1000000 }; // 1ms
+				nanosleep( &t, NULL );
 			}
 			else
 			{
