@@ -1,6 +1,7 @@
 /*
-* playlist.cc
+* playlist.cc -- SMIL implementation
 * Copyright (C) 2000 Arne Schirmacher <arne@schirmacher.de>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -43,14 +44,19 @@ using std::map;
 #include <libxml/tree.h>
 #include <pthread.h>
 #include <unistd.h>
+#include <sys/stat.h>
+#include <sys/types.h>
+#include <dirent.h>
 
 // local includes
 #include "playlist.h"
 #include "error.h"
 #include "filehandler.h"
 #include "frame.h"
+#include "stringutils.h"
 
 const xmlChar* SMIL20_NAMESPACE_HREF = reinterpret_cast< const xmlChar* >( "http://www.w3.org/2001/SMIL20/Language" );
+const string KINO_AUTOSAVE_DIR = string( getenv("HOME") ) + string( "/.kino-history" );
 
 /** The file map implementation - this class is responsible for maintaining the list of
 	open files associated to a project.
@@ -101,64 +107,6 @@ FileMap *GetFileMap( )
 	return thismap;
 }
 
-/** Split the string by delimiter and return each resultant string in items.
- 
-    Note that the items vector is not cleaned betweeen iterations.
-*/
-
-int string_utils::split( const string &input, const string &delimiter, vector< string > &items, const bool clean )
-{
-	int delimiter_size = delimiter.size();
-	int input_size = input.size();
-
-	// Find the first delimiter
-	int position = 0;
-	int end_position = input.find( delimiter, 0 );
-
-	// While we have a valid position
-	while ( end_position >= position )
-	{
-		// Obtain the substr and push if valid
-		string s = input.substr( position, end_position - position );
-		if ( !clean || s.size() > 0 )
-			items.push_back( s );
-
-		// Find the next delimiter
-		position = end_position + delimiter_size;
-		end_position = input.find( delimiter, position );
-	}
-
-	// Obtain the substr of what's left and push if valid
-	string s = input.substr( position, input_size - position );
-	if ( !clean || s.size() > 0 )
-		items.push_back( s );
-
-	// Return the number of items found
-	return items.size();
-}
-
-/** Join the contents of items with the given delimiter.
- 
-    Note that the no leading or trailing delimters are output.
-*/
-
-string string_utils::join( vector< string >&items, const string &delimiter )
-{
-	string output( "" );
-
-	// Loop through the items
-	for ( vector< string >::iterator item = items.begin( ); item != items.end( ); item ++ )
-	{
-		if ( item == items.begin() )
-			output += *item;
-		else
-			output += delimiter + *item;
-	}
-
-	return string( output );
-}
-
-
 /** Join the file to the directory.
  
     It is assumed that if the input file is not an absolute path (ie: starting with a /)
@@ -179,15 +127,15 @@ string directory_utils::join_file_to_directory( const string directory, const st
 	{
 		char path[ PATH_MAX ];
 		getcwd( path, PATH_MAX );
-		string_utils::split( path, "/", items );
+		StringUtils::split( path, "/", items );
 	}
 
 	// Now add the directory if file is not absolute
 	if ( file[ 0 ] != '/' )
-		string_utils::split( directory, "/", items );
+		StringUtils::split( directory, "/", items );
 
 	// Split the file and append to the directory info
-	string_utils::split( file, "/", items );
+	StringUtils::split( file, "/", items );
 
 	// iterate through the items vector
 	for ( vector< string >::iterator item = items.begin( ); item != items.end( ); )
@@ -212,7 +160,7 @@ string directory_utils::join_file_to_directory( const string directory, const st
 		}
 	}
 
-	return "/" + string_utils::join( items, "/" );
+	return "/" + StringUtils::join( items, "/" );
 }
 
 /** Obtain the directory from the file.
@@ -256,8 +204,8 @@ string directory_utils::get_relative_path_to_file( const string &directory, cons
 	string absolute = join_file_to_directory( directory, file );
 	vector < string > directory_items;
 	vector < string > absolute_items;
-	string_utils::split( absolute, "/", absolute_items );
-	string_utils::split( directory, "/", directory_items );
+	StringUtils::split( absolute, "/", absolute_items );
+	StringUtils::split( directory, "/", directory_items );
 
 	vector < string >::iterator directory_item;
 	vector < string >::iterator absolute_item;
@@ -277,7 +225,7 @@ string directory_utils::get_relative_path_to_file( const string &directory, cons
 		output += "../";
 
 	// Now simply join what's left in absolute to output and return
-	output += string_utils::join( absolute_items, "/" );
+	output += StringUtils::join( absolute_items, "/" );
 	return output;
 }
 
@@ -288,7 +236,7 @@ string directory_utils::expand_directory( const string directory )
 {
 	string output;
 	vector <string> items;
-	string_utils::split( directory, "/", items );
+	StringUtils::split( directory, "/", items );
 	vector< string >::iterator item = items.begin( );
 
 	if ( item != items.end( ) && *item == "~" )
@@ -722,6 +670,98 @@ static bool convertEli( xmlNodePtr node, void *p, bool *freed )
 	return false;
 }
 
+class SrtContext
+{
+public:
+	ofstream file;
+	unsigned counter;
+	xmlChar* title;
+	xmlChar* abstract;
+	xmlChar* src;
+	unsigned begin;
+	unsigned duration;
+
+	SrtContext( const char* filename ) :
+		file(filename), counter(0), title(NULL), abstract(NULL), src(NULL),
+		begin(0), duration(0)
+	{
+	}
+
+	~SrtContext()
+	{
+		printEntry();
+		file.close();
+	}
+
+	void printEntry(void)
+	{
+		if ( title || abstract )
+		{
+			Frame *frame = GetFramePool( )->GetFrame( );
+			FileHandler *mediaFile = GetFileMap()->GetMap() [ string( ( const char* ) src ) ];
+			SMIL::MediaClippingTime time;
+
+			mediaFile->GetFrame( *frame, 0 );
+			time.setFramerate( frame->GetFrameRate() );
+			GetFramePool( )->DoneWithFrame( frame );
+			string beginString = time.parseFramesToString( begin, SMIL::Time::TIME_FORMAT_CLOCK );
+			string durationString = time.parseFramesToString( duration - begin, SMIL::Time::TIME_FORMAT_CLOCK );
+			begin = 0;
+
+			file << ++counter << endl;
+			file << StringUtils::replaceAll( beginString, ".", "," );
+			file << " --> " << StringUtils::replaceAll( durationString, ".", "," ) << endl;
+			if ( title )
+				file << title << endl;
+			if ( abstract )
+				file << abstract << endl;
+			file << endl;
+		}
+		if ( src )
+		{
+			xmlFree( src );
+			src = NULL;
+		}
+		if ( title )
+		{
+			xmlFree( title );
+			title = NULL;
+		}
+		if ( abstract )
+		{
+			xmlFree( abstract );
+			abstract = NULL;
+		}
+	}
+};
+
+static bool convertSrt( xmlNodePtr node, void *data, bool *freed )
+{
+	SrtContext* srt = static_cast< SrtContext* >( data );
+	
+	if ( xmlStrcmp( node->name, ( const xmlChar* ) "seq" ) == 0 )
+	{
+		srt->printEntry();
+		srt->title = xmlGetProp( node, ( const xmlChar * ) "title" );
+		srt->abstract = xmlGetProp( node, ( const xmlChar * ) "abstract" );
+	}
+	else if ( xmlStrcmp( node->name, ( const xmlChar* ) "video" ) == 0 )
+	{
+		xmlChar* clipBegin = xmlGetProp( node, ( const xmlChar* ) "clipBegin" );
+		xmlChar* clipEnd = xmlGetProp( node, ( const xmlChar* ) "clipEnd" );
+
+		srt->src = xmlGetProp( node, ( const xmlChar* ) "src" );
+		if ( !srt->begin && ( srt->title || srt->abstract ) )
+			srt->begin = srt->duration;
+		srt->duration += atoi( ( const char* ) clipEnd ) - atoi( ( const char* ) clipBegin ) + 1;
+
+		xmlFree( clipBegin );
+		xmlFree( clipEnd );
+	}
+
+	return false;
+}
+
 static bool convertFramesToSmilTime( xmlNodePtr node, void *data, bool *freed )
 {
 	if ( xmlStrcmp( node->name, ( const xmlChar* ) "video" ) == 0 )
@@ -946,7 +986,7 @@ char* PlayList::GetFileNameOfFrame( int frameNum ) const
 	data.absFrame = frameNum;
 
 	parse( GetBody(), findFile, &data );
-	return data.fileName;
+	return strdup( data.fileName );
 }
 
 
@@ -1314,6 +1354,30 @@ bool PlayList::JoinScenesAt( int frameNum )
 			lastchild = xmlAddNextSibling( lastchild, ptr );
 			ptr = next;
 		}
+		// Merge the metadata properties
+		for ( xmlAttr* attr = scene2end.sequence->properties; attr; attr = attr->next )
+		{
+			const char *valueB = ( const char* ) xmlGetProp( attr->parent, ( const xmlChar * ) attr->name );
+			// Only if the value is meaningful
+			if ( valueB && strcmp( valueB, "" ) )
+			{
+				// See if the property exists on clip A
+				const char *valueA = ( const char* ) xmlGetProp( scene1.sequence, ( const xmlChar * ) attr->name );
+				if ( valueA && strcmp( valueA, "" ) )
+				{
+					xmlChar* value = new xmlChar[ strlen(valueA) + strlen(valueB) + 1 ];
+					strcpy( (char*) value, (const char*) valueA );
+					strcat( (char*) value, " " );
+					strcat( (char*) value, (const char*) valueB );
+					xmlSetProp( scene1.sequence, attr->name, ( const xmlChar * ) value );
+					delete[] value;
+				}
+				else
+				{
+					xmlSetProp( scene1.sequence, attr->name, ( const xmlChar * ) valueB );
+				}
+			}
+		}
 		xmlUnlinkNode( scene2end.sequence );
 		xmlFreeNode( scene2end.sequence );
 		RefreshCount( );
@@ -1748,16 +1812,18 @@ bool PlayList::LoadMediaObject( char *filename )
 	string index = directory_utils::get_absolute_path_to_file( "", ( char * ) filename );
 	if ( GetFileMap() ->GetMap().find( index ) == GetFileMap() ->GetMap().end( ) )
 	{
-		if ( strncasecmp( strrchr( filename, '.' ), ".avi", 4 ) == 0 )
-			mediaFile = new AVIHandler();
-		else if ( strncasecmp( strrchr( filename, '.' ), ".dv", 3 ) == 0 ||
-		          strncasecmp( strrchr( filename, '.' ), ".dif", 4 ) == 0 )
-			mediaFile = new RawHandler();
+		if ( strrchr( filename, '.' ) )
+		{
+			if ( strncasecmp( strrchr( filename, '.' ), ".avi", 4 ) == 0 )
+				mediaFile = new AVIHandler();
+			else if ( strncasecmp( strrchr( filename, '.' ), ".dv", 3 ) == 0 ||
+					strncasecmp( strrchr( filename, '.' ), ".dif", 4 ) == 0 )
+				mediaFile = new RawHandler();
 #ifdef HAVE_LIBQUICKTIME
-
-		else if ( strncasecmp( strrchr( filename, '.' ), ".mov", 4 ) == 0 )
-			mediaFile = new QtHandler();
+			else if ( strncasecmp( strrchr( filename, '.' ), ".mov", 4 ) == 0 )
+				mediaFile = new QtHandler();
 #endif
+		}
 
 		if ( mediaFile == NULL )
 			return false;
@@ -1776,7 +1842,7 @@ bool PlayList::LoadMediaObject( char *filename )
 	seq = xmlNewNode( NULL, ( const xmlChar* ) "seq" );
 	xmlAddChild( GetBody(), seq );
 	node = xmlNewNode( NULL, ( const xmlChar* ) "video" );
-	xmlNewProp( node, ( const xmlChar* ) "src", ( const xmlChar* ) filename );
+	xmlNewProp( node, ( const xmlChar* ) "src", ( const xmlChar* ) index.c_str() );
 	xmlNewProp( node, ( const xmlChar* ) "clipBegin", ( const xmlChar* ) "0" );
 	sb << framesInFile - 1 << ends;
 	xmlNewProp( node, ( const xmlChar* ) "clipEnd", ( const xmlChar* ) sb.str().c_str() );
@@ -1801,7 +1867,12 @@ bool PlayList::LoadPlayList( char *filename )
 	xmlNodePtr node;
 
 	xmlFreeDoc( doc );
-	fail_null( doc = xmlParseFile( filename ) );
+	doc = xmlParseFile( filename );
+	if ( !doc )
+	{
+		cerr << "file does not exist or failed to parse XML" << endl;
+		return false;
+	}
 
 	node = xmlDocGetRootElement( doc );
 	if ( node == NULL )
@@ -1843,17 +1914,22 @@ bool PlayList::LoadPlayList( char *filename )
 	else
 	{
 		// We need to insert a body element into the legacy document
-		xmlNodePtr body = xmlNewNode( NULL, ( const xmlChar * ) "body" );
 		xmlNodePtr root = xmlDocGetRootElement( doc );
 		xmlNodePtr seq = root->children;
-		while ( seq )
+		if ( !seq )
+			return false;
+		if ( xmlStrcmp( seq->name, ( const xmlChar* ) "seq" ) == 0 )
 		{
-			xmlNodePtr next = seq->next;
-			xmlUnlinkNode( seq );
-			xmlAddChild( body, seq );
-			seq = next;
+			xmlNodePtr body = xmlNewNode( NULL, ( const xmlChar * ) "body" );
+			while ( seq )
+			{
+				xmlNodePtr next = seq->next;
+				xmlUnlinkNode( seq );
+				xmlAddChild( body, seq );
+				seq = next;
+			}
+			xmlAddChild( root, body );
 		}
-		xmlAddChild( root, body );
 		dirty = true;
 	}
 
@@ -1894,14 +1970,23 @@ bool PlayList::SavePlayList( char *filename, bool isLegacyFormat )
 	xmlNodePtr root = xmlNewNode( NULL, ( const xmlChar* ) "smil" );
 	xmlNewNs( root, ( const xmlChar* ) SMIL20_NAMESPACE_HREF, ( const xmlChar* ) NULL );
 	xmlDocSetRootElement( copy_doc, root );
-	parse( this->GetBody(), clone, &root );
 
 	if ( isLegacyFormat )
+	{
+		// Clone without body
+		parse( this->GetBody()->children, clone, &root );	
+
 		// Add legacy namespace declaration
 		xmlNewNs( xmlDocGetRootElement( copy_doc ), SMIL20_NAMESPACE_HREF, ( const xmlChar* ) "smil2" );
+	}
 	else
+	{
+		// Clone including body
+		parse( this->GetBody(), clone, &root );
+
 		// Convert frame numbers used internally to SMIL time values.
 		parse( copy_doc->children, convertFramesToSmilTime, NULL );
+	}
 
 	if ( Preferences::getInstance().relativeSave )
 	{
@@ -2006,6 +2091,23 @@ bool PlayList::SavePlayListEli( char * filename, bool isPAL )
 }
 
 
+bool PlayList::SavePlayListSrt( const char* filename )
+{
+	SrtContext srt( filename );
+
+	/* Open a file */
+	if ( !srt.file )
+	{
+		return false;
+	}
+	if ( doc )
+	{
+		parse( GetBody(), convertSrt, &srt );
+	}
+	
+	return !srt.file.bad();
+}
+
 
 /** Recursively deletes unnecessary items from our XML tree.
  
@@ -2286,7 +2388,7 @@ EditorBackup::~EditorBackup()
 	}
 }
 
-void EditorBackup::Store( PlayList *playlist )
+void EditorBackup::Store( PlayList *playlist, bool isPersisted )
 {
 	cerr << ">>> Received playlist to store at position " << position + 1 << endl;
 
@@ -2349,6 +2451,45 @@ void EditorBackup::Store( PlayList *playlist )
 	{
 		cerr << ">>>> Unknown condition - position = " << position << " size = " << backups.size() << endl;
 	}
+
+	// Update persistent storage
+	if ( isPersisted )
+	{
+		// First, delete contents of private directory in $HOME
+		string directory = KINO_AUTOSAVE_DIR;
+		mkdir( directory.c_str(), 0700 );
+		DIR* dir = opendir( directory.c_str( ) );
+		struct dirent* entry;
+		if ( dir )
+		{
+			while ( ( entry = readdir( dir ) ) != NULL )
+			{
+				if ( entry->d_name[0] != '.' )
+				{
+					ostringstream sb;
+					sb << directory << "/" << entry->d_name << ends;
+					unlink( sb.str().c_str() );
+				}
+			}
+			closedir( dir );
+	
+			// Second, write each dirty backup
+			// absolute filenames is faster and more appropriate here
+			bool isRelative = Preferences::getInstance().relativeSave;
+			Preferences::getInstance().relativeSave = false;
+			for ( size_t i = 0; i < backups.size(); ++i )
+			{
+				if ( backups[i]->IsDirty() )
+				{
+					ostringstream sb;
+					sb << directory << "/" << i << ".xml" << ends;
+					// save in legacy format for speed
+					backups[i]->SavePlayList( const_cast<char*>( sb.str().c_str() ), true );
+				}
+			}
+			Preferences::getInstance().relativeSave = isRelative;
+		}
+	}
 }
 
 void EditorBackup::Undo( PlayList *playlist )
@@ -2422,3 +2563,40 @@ EditorBackup *GetEditorBackup( )
 	static EditorBackup * backup = new EditorBackup( );
 	return backup;
 }
+
+bool EditorBackup::Restore( PlayList * playlist )
+{
+	string directory = KINO_AUTOSAVE_DIR;
+	DIR* dir = opendir( directory.c_str( ) );
+	struct dirent* entry;
+	bool result = false;
+
+	if ( dir )
+	{
+		std::vector<int> names;
+		while ( ( entry = readdir( dir ) ) != NULL )
+		{
+			if ( entry->d_name[0] != '.' )
+				names.push_back( atoi( entry->d_name ) );
+		}
+		closedir( dir );
+
+		if ( !names.empty() )
+		{
+			std::sort( names.begin(), names.end() );
+			Clear();
+			PlayList pl;
+			for ( size_t i = 0; i < names.size(); ++i )
+			{
+				ostringstream sb;
+				sb << directory << "/" << names[i] << ".xml" << ends;
+				if ( pl.LoadPlayList( const_cast<char*>( sb.str().c_str() ) ) )
+					Store( &pl, false );
+			}
+			++position;
+			Undo( playlist );
+			result = true;
+		}
+	}
+	return result;
+}
