@@ -1,5 +1,5 @@
 /*
-* Copyright (C) 2001 Dan Dennedy <dan@dennedy.org>
+* Copyright (C) 2001-2007 Dan Dennedy <dan@dennedy.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -23,6 +23,7 @@
 
 #include "preferences.h"
 #include "kino_common.h"
+#include "stringutils.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -71,7 +72,6 @@ Preferences &Preferences::getInstance()
 {
 	if ( instance == NULL )
 	{
-		std::cerr << "> Creating Preferences" << std::endl;
 		instance = new Preferences;
 	}
 	return *instance;
@@ -133,10 +133,10 @@ Preferences::Preferences() : phyID( -1 )
 	if ( f )
 		fclose( f );
 	else
+		// Try to load old config if nothing yet found
 		filename = string( getenv( "HOME" ) ) + string( "/.gnome2/kino" );
 	
 	// Read the config file
-	cerr << "Loading preferences from \"" << filename << "\"" << endl;
 	config = g_key_file_new();
 	g_key_file_load_from_file( config, filename.c_str(), G_KEY_FILE_KEEP_COMMENTS, NULL );
 
@@ -149,7 +149,7 @@ Preferences::Preferences() : phyID( -1 )
 	get( fileFormat, "fileFormat", key );
 	get( autoSplit, "autoSplit", "true" );
 	get( timeStamp, "timeStamp", "false" );
-	get( frames, "frames", "7000" );
+	get( frames, "frames", "0" );
 	get( every, "every", "1" );
 	get( interface, "interface", "0" );
 	get( channel, "channel", "63" );
@@ -162,19 +162,18 @@ Preferences::Preferences() : phyID( -1 )
 	get( cip_n, "cip_increment", "0" );
 	get( cip_d, "cip_threshold", "0" );
 	get( syt_offset, "syt_offset", "0" );
-	get( preview_capture, "preview_capture", "false" );
-	get( dropFrame, "dropFrame", "false" );
+	get( preview_capture, "preview_capture", "true" );
+	get( dropFrame, "dropFrame", "true" );
 	get( audioDevice, sizeof(audioDevice), "audioDevice", "default" );
 	get( enableV4L, "enableV4L", "false" );
 	get( disableKeyRepeat, "disableKeyRepeat", "false" );
 	get( audioRendering, "audioRendering", "0" );
-	get( previewRendering, "previewRendering", "1" );
 	get( maxUndos, "maxUndos", "50" );
 	get( dvCaptureBuffers, "dvCaptureBuffers", "50" );
 	get( dvExportBuffers, "dvExportBuffers", "10" );
 	get( dvDecoderClampLuma, "dvDecoderClampLuma", "false" );
 	get( dvDecoderClampChroma, "dvDecoderClampChroma", "false" );
-	get( maxFileSize, "maxFileSize", "2000" );
+	get( maxFileSize, "maxFileSize", "0" );
 	get( audioScrub, "audioScrub", "false" );
 	get( v4lVideoDevice, sizeof(v4lVideoDevice), "v4lVideoDevice", "/dev/video0" );
 	get( v4lAudioDevice, sizeof(v4lAudioDevice), "v4lAudioDevice", "/dev/dsp" );
@@ -182,35 +181,33 @@ Preferences::Preferences() : phyID( -1 )
 	get( v4lAudio, sizeof(v4lAudio), "v4lAudio", "32000" );
 	get( isOpenDML, "isOpenDML", "false" );
 	get( defaultDirectory, sizeof(defaultDirectory), "defaultDirectory", "~/" );
-	get( displayExtract, "displayExtract", "0" );
-	get( relativeSave, "relativeSave", "false" );
-	get( dvCaptureDriver, "dvCaptureDriver", "0" );
+	get( displayExtract, "displayExtract", "1" );
+	get( relativeSave, "relativeSave", "true" );
 	get( dvCaptureDevice, sizeof(dvCaptureDevice), "dvCaptureDevice", "/dev/dv1394/0" );
 	get( dv1394Preview, "dv1394Preview", "true" );
 	get( dvExportDevice, sizeof(dvExportDevice), "dvExportDevice", "/dev/dv1394/0" );
 	get( avcPollIntervalMs, "avcPollIntervalMs", "200" );
 	get( dvExportPrerollSec, "dvExportPrerollSec", "4" );
-	get( dvTwoPassEncoder, "dvTwoPassEncoder", "false" );
+	get( dvTwoPassEncoder, "dvTwoPassEncoder", "true" );
 	get( windowWidth, "windowWidth", "-1" );
 	get( windowHeight, "windowHeight", "-1" );
 	get( storyboardPosition, "storyboardPosition", "-1" );
 	get( previewSize, "previewSize", "0" );
-	get( timeFormat, "timeFormat", "1" );
+	get( timeFormat, "timeFormat", "3" );
 	get( trimModeInsert, "trimModeInsert", "false" );
-	get( enablePublish, "enablePublish", "false" );
+	get( enablePublish, "enablePublish", "true" );
 	get( expandStoryboard, "expandStoryboard", "false" );
 	get( newProjectURI, sizeof(newProjectURI), "newProjectURI", "" );
 	get( newProjectXPath, sizeof(newProjectXPath), "newProjectXPath", "" );
 	get( metaNames, sizeof(metaNames), "metaNames", "*title,author,copyright,id,abstract" );
 	get( enableAVC, "enableAVC", "true" );
 
-	get( exportMjpegFormat, "exportMjpegFormat", "0" );
+	get( exportMjpegFormat, "exportMjpegFormat", "6" );
 	get( exportMjpegAspect, "exportMjpegAspect", "0" );
 	get( exportMjpegDeinterlace, "exportMjpegDeinterlace", "0" );
 	get( exportMjpegSceneSplit, "exportMjpegSceneSplit", "false" );
 	get( exportMjpegCleanup, "exportMjpegCleanup", "true" );
-	get( exportMjpegOutputDVDAuthorXML, "exportMjpegOutputDVDAuthorXML", "false" );
-	get( exportMjpegDvdTool, "exportMjpegDvdTool", "0" );
+	get( exportMjpegDvdTool, "exportMjpegDvdTool", "1" );
 	get( exportMjpegVideoPipe, sizeof(exportMjpegVideoPipe), "exportMjpegVideoPipe", "mpeg2enc -v 0" );
 	get( exportMjpegAudioPipe, sizeof(exportMjpegAudioPipe), "exportMjpegAudioPipe", "mp2enc -v 0" );
 	get( exportMjpegMultiplex, sizeof(exportMjpegMultiplex), "exportMjpegMultiplex", "mplex -v 0" );
@@ -221,7 +218,7 @@ Preferences::Preferences() : phyID( -1 )
 	vector< string > metaNamesList;
 	vector< string >::iterator metaNamesIter;
 	
-	string_utils::split( metaNames, ",", metaNamesList );
+	StringUtils::split( metaNames, ",", metaNamesList );
 	for ( metaNamesIter = metaNamesList.begin(); metaNamesIter != metaNamesList.end(); ++metaNamesIter )
 	
 	{
@@ -232,19 +229,22 @@ Preferences::Preferences() : phyID( -1 )
 			metaname.erase( 0, 1 );
 
 		str << "metaValues_" << metaname;
-		get( key, 4096, str.str().c_str(), "*" );
+		if ( metaname == "copyright" )
+			get( key, 4096, str.str().c_str(), "No license (All rights reserved)=-1,Creative Commons Attribution=1,Creative Commons Attribution-NoDerivs=2,Creative Commons Attribution-NonCommercial-NoDerivs=3,Creative Commons Attribution-NonCommercial=4,Creative Commons Attribution-NonCommercial-ShareAlike=5,Creative Commons Attribution-ShareAlike=6,Public Domain=7,*" );
+		else
+			get( key, 4096, str.str().c_str(), "*" );
 		{
 			vector< string > metaValuesSimpleList;
 			vector< string >::iterator iter;
 			vector< pair< string, string > > metaValuesList;
 
-			string_utils::split( key, ",", metaValuesSimpleList );
+			StringUtils::split( key, ",", metaValuesSimpleList );
 			for ( iter = metaValuesSimpleList.begin(); iter != metaValuesSimpleList.end(); ++iter )
 			{
 				vector< string > labelValueVector;
 				pair< string, string > labelValuePair;
 				
-				string_utils::split( *iter, "=", labelValueVector );
+				StringUtils::split( *iter, "=", labelValueVector );
 				if ( labelValueVector.size() > 1 )
 					labelValuePair = make_pair( labelValueVector[0], labelValueVector[1] );
 				else
@@ -256,6 +256,7 @@ Preferences::Preferences() : phyID( -1 )
 	}
 
 	JogShuttleInit();
+	RecentFilesInit();
 	
 	Save();
 	delete[] key;
@@ -646,8 +647,6 @@ void Preferences::set( const char *name, bool value )
 void
 Preferences::Save()
 {
-	cerr << "Saving preferences." << endl;
-
 	set( "defaultNormalisation", defaultNormalisation );
 	set( "defaultAudio", defaultAudio );
 	set( "defaultAspect", defaultAspect );
@@ -693,7 +692,6 @@ Preferences::Save()
 	set( "enableV4L", enableV4L );
 	set( "disableKeyRepeat", disableKeyRepeat );
 	set( "audioRendering", audioRendering );
-	set( "previewRendering", previewRendering );
 	set( "maxUndos", maxUndos );
 	set( "dvCaptureBuffers", dvCaptureBuffers );
 	set( "dvExportBuffers", dvExportBuffers );
@@ -710,7 +708,6 @@ Preferences::Save()
 	set( "displayExtract", displayExtract );
 	set( "relativeSave", relativeSave );
 
-	set( "dvCaptureDriver", dvCaptureDriver );
 	set( "dvCaptureDevice", dvCaptureDevice );
 	set( "dv1394Preview", dv1394Preview );
 	set( "dvExportDevice", dvExportDevice );
@@ -735,12 +732,13 @@ Preferences::Save()
 	set( "exportMjpegAspect", exportMjpegAspect );
 	set( "exportMjpegSceneSplit", exportMjpegSceneSplit );
 	set( "exportMjpegCleanup", exportMjpegCleanup );
-	set( "exportMjpegOutputDVDAuthorXML", exportMjpegOutputDVDAuthorXML );
 	set( "exportMjpegDvdTool", exportMjpegDvdTool );
 	set( "exportMjpegVideoPipe", exportMjpegVideoPipe );
 	set( "exportMjpegAudioPipe", exportMjpegAudioPipe );
 	set( "exportMjpegMultiplex", exportMjpegMultiplex );
 
+	RecentFilesSave();
+
 	// Persist
 	string filename = kino_config_file();
 	FILE *f = fopen( filename.c_str(), "w" );
@@ -758,3 +756,54 @@ Preferences::Save()
 	unlink( filename.c_str() );
 	errno = 0;
 }
+
+void Preferences::RecentFilesInit()
+{
+	char* key = new char[64];
+	char* value = new char[ PATH_MAX + NAME_MAX ];
+
+	for ( int i = 0; i < 5 ; i++ )
+	{
+		sprintf( key, "recentFile%i", i );
+		value[0] = 0;
+		get( value, PATH_MAX + NAME_MAX, key, "" );
+		if ( strcmp( value, "" ) )
+			recentFiles.push_back( value );
+	}
+	
+	delete[] key;
+	delete[] value;
+}
+
+void Preferences::RecentFilesSave()
+{
+	vector<string>::iterator i;
+	int count = 0;
+
+	for ( i = recentFiles.begin(); i != recentFiles.end(); i++, count++ )
+	{
+		char key[64];
+		sprintf( key, "recentFile%i", count );
+		set( key, (*i).c_str() );
+	}
+}
+
+void Preferences::addRecentFile( string filename )
+{
+	vector<std::string>::iterator i;
+	for ( i = recentFiles.begin(); i != recentFiles.end(); i++ )
+	{
+		if ( (*i) == filename )
+		{
+			recentFiles.erase( i );
+			recentFiles.push_back( filename );
+			break;
+		}
+	}
+	if ( i == recentFiles.end() )
+	{
+		if ( recentFiles.size() > 4 )
+			recentFiles.erase( recentFiles.begin() );
+		recentFiles.push_back( filename );
+	}
+}
